@@ -1,7 +1,7 @@
 /* GDK - The GIMP Drawing Kit
  * gdkdisplay.c
- * 
- * Copyright 2001 Sun Microsystems Inc. 
+ *
+ * Copyright 2001 Sun Microsystems Inc.
  *
  * Erwann Chenede <erwann.chenede@sun.com>
  *
@@ -31,14 +31,25 @@
 #include "gdkclipboardprivate.h"
 #include "gdkdeviceprivate.h"
 #include "gdkdisplaymanagerprivate.h"
-#include "gdkframeclockidleprivate.h"
+#include "gdkdmabufeglprivate.h"
+#include "gdkdmabufformatsbuilderprivate.h"
+#include "gdkdmabufformatsprivate.h"
+#include "gdkdmabuftextureprivate.h"
 #include "gdkeventsprivate.h"
+#include "gdkframeclockidleprivate.h"
 #include "gdkglcontextprivate.h"
 #include "gdkmonitorprivate.h"
+#include "gdkrectangle.h"
+#include "gdkvulkancontextprivate.h"
 
 #ifdef HAVE_EGL
 #include <epoxy/egl.h>
 #endif
+
+#ifdef HAVE_SYS_SYSMACROS_H
+#include <sys/sysmacros.h>
+#endif
+
 #include <math.h>
 #include <stdlib.h>
 
@@ -66,7 +77,9 @@ enum
   PROP_0,
   PROP_COMPOSITED,
   PROP_RGBA,
+  PROP_SHADOW_WIDTH,
   PROP_INPUT_SHAPES,
+  PROP_DMABUF_FORMATS,
   LAST_PROP
 };
 
@@ -99,6 +112,7 @@ struct _GdkDisplayPrivate {
 
   guint rgba : 1;
   guint composited : 1;
+  guint shadow_width: 1;
   guint input_shapes : 1;
 
   GdkDebugFlags debug_flags;
@@ -132,10 +146,18 @@ gdk_display_get_property (GObject    *object,
       g_value_set_boolean (value, gdk_display_is_rgba (display));
       break;
 
+    case PROP_SHADOW_WIDTH:
+      g_value_set_boolean (value, gdk_display_supports_shadow_width (display));
+      break;
+
     case PROP_INPUT_SHAPES:
       g_value_set_boolean (value, gdk_display_supports_input_shapes (display));
       break;
 
+    case PROP_DMABUF_FORMATS:
+      g_value_set_boxed (value, gdk_display_get_dmabuf_formats (display));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -175,7 +197,7 @@ gdk_display_default_rate_egl_config (GdkDisplay *display,
 
   return distance;
 }
-    
+
 static GdkSeat *
 gdk_display_real_get_default_seat (GdkDisplay *display)
 {
@@ -227,6 +249,18 @@ gdk_display_class_init (GdkDisplayClass *class)
                           TRUE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GdkDisplay:shadow-width: (attributes org.gtk.Property.get=gdk_display_supports_shadow_width)
+   *
+   * %TRUE if the display supports extensible frames.
+   *
+   * Since: 4.14
+   */
+  props[PROP_SHADOW_WIDTH] =
+    g_param_spec_boolean ("shadow-width", NULL, NULL,
+                          TRUE,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   /**
    * GdkDisplay:input-shapes: (attributes org.gtk.Property.get=gdk_display_supports_input_shapes)
    *
@@ -237,6 +271,18 @@ gdk_display_class_init (GdkDisplayClass *class)
                           TRUE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GdkDisplay:dmabuf-formats: (attributes org.gtk.Property.get=gdk_display_get_dmabuf_formats)
+   *
+   * The dma-buf formats that are supported on this display
+   *
+   * Since: 4.14
+   */
+  props[PROP_DMABUF_FORMATS] =
+    g_param_spec_boxed ("dmabuf-formats", NULL, NULL,
+                        GDK_TYPE_DMABUF_FORMATS,
+                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   /**
@@ -322,7 +368,7 @@ static void
 free_pointer_info (GdkPointerSurfaceInfo *info)
 {
   g_clear_object (&info->surface_under_pointer);
-  g_slice_free (GdkPointerSurfaceInfo, info);
+  g_free (info);
 }
 
 static void
@@ -363,6 +409,7 @@ gdk_display_init (GdkDisplay *display)
 
   priv->composited = TRUE;
   priv->rgba = TRUE;
+  priv->shadow_width = TRUE;
   priv->input_shapes = TRUE;
 }
 
@@ -371,17 +418,40 @@ gdk_display_dispose (GObject *object)
 {
   GdkDisplay *display = GDK_DISPLAY (object);
   GdkDisplayPrivate *priv = gdk_display_get_instance_private (display);
+  gsize i;
+
+  for (i = 0; i < G_N_ELEMENTS (display->dmabuf_downloaders); i++)
+    {
+      if (display->dmabuf_downloaders[i] == NULL)
+        continue;
+
+      gdk_dmabuf_downloader_close (display->dmabuf_downloaders[i]);
+      g_clear_object (&display->dmabuf_downloaders[i]);
+    }
 
   _gdk_display_manager_remove_display (gdk_display_manager_get (), display);
 
   g_queue_clear (&display->queued_events);
 
+  g_clear_pointer (&display->egl_dmabuf_formats, gdk_dmabuf_formats_unref);
+  g_clear_pointer (&display->egl_external_formats, gdk_dmabuf_formats_unref);
+#ifdef GDK_RENDERING_VULKAN
+  if (display->vk_dmabuf_formats)
+    {
+      gdk_display_unref_vulkan (display);
+      g_assert (display->vk_dmabuf_formats == NULL);
+    }
+#endif
+
   g_clear_object (&priv->gl_context);
 #ifdef HAVE_EGL
   g_clear_pointer (&priv->egl_display, eglTerminate);
 #endif
   g_clear_error (&priv->gl_error);
 
+  for (GList *l = display->seats; l; l = l->next)
+    g_object_run_dispose (G_OBJECT (l->data));
+
   G_OBJECT_CLASS (gdk_display_parent_class)->dispose (object);
 }
 
@@ -399,6 +469,8 @@ gdk_display_finalize (GObject *object)
 
   g_list_free_full (display->seats, g_object_unref);
 
+  g_clear_pointer (&display->dmabuf_formats, gdk_dmabuf_formats_unref);
+
   G_OBJECT_CLASS (gdk_display_parent_class)->finalize (object);
 }
 
@@ -418,10 +490,10 @@ gdk_display_close (GdkDisplay *display)
   if (!display->closed)
     {
       display->closed = TRUE;
-      
+
       g_signal_emit (display, signals[CLOSED], 0, FALSE);
       g_object_run_dispose (G_OBJECT (display));
-      
+
       g_object_unref (display);
     }
 }
@@ -468,8 +540,7 @@ gdk_display_get_event (GdkDisplay *display)
  * @display: a `GdkDisplay`
  * @event: (transfer none): a `GdkEvent`
  *
- * Appends the given event onto the front of the event
- * queue for @display.
+ * Adds the given event to the event queue for @display.
  *
  * Deprecated: 4.10: This function is only useful in very
  * special situations and should not be used by applications.
@@ -808,7 +879,7 @@ _gdk_display_end_device_grab (GdkDisplay *display,
       grab->implicit_ungrab = implicit;
       return l->next == NULL;
     }
-  
+
   return FALSE;
 }
 
@@ -834,7 +905,7 @@ _gdk_display_get_pointer_info (GdkDisplay *display,
 
   if (G_UNLIKELY (!info))
     {
-      info = g_slice_new0 (GdkPointerSurfaceInfo);
+      info = g_new0 (GdkPointerSurfaceInfo, 1);
       g_hash_table_insert (display->pointers_info, device, info);
     }
 
@@ -1144,9 +1215,9 @@ _gdk_display_get_next_serial (GdkDisplay *display)
  * Indicates to the GUI environment that the application has
  * finished loading, using a given identifier.
  *
- * GTK will call this function automatically for [class@Gtk.Window]
+ * GTK will call this function automatically for [GtkWindow](../gtk4/class.Window.html)
  * with custom startup-notification identifier unless
- * [method@Gtk.Window.set_auto_startup_notification]
+ * [gtk_window_set_auto_startup_notification()](../gtk4/method.Window.set_auto_startup_notification.html)
  * is called to disable that feature.
  *
  * Deprecated: 4.10: Using [method@Gdk.Toplevel.set_startup_id] is sufficient
@@ -1196,21 +1267,6 @@ _gdk_display_unpause_events (GdkDisplay *display)
   display->event_pause_count--;
 }
 
-GdkSurface *
-gdk_display_create_surface (GdkDisplay     *display,
-                            GdkSurfaceType  surface_type,
-                            GdkSurface     *parent,
-                            int             x,
-                            int             y,
-                            int             width,
-                            int             height)
-{
-  return GDK_DISPLAY_GET_CLASS (display)->create_surface (display,
-                                                          surface_type,
-                                                          parent,
-                                                          x, y, width, height);
-}
-
 /*< private >
  * gdk_display_get_keymap:
  * @display: the `GdkDisplay`
@@ -1227,6 +1283,75 @@ gdk_display_get_keymap (GdkDisplay *display)
   return GDK_DISPLAY_GET_CLASS (display)->get_keymap (display);
 }
 
+/*<private>
+ * gdk_display_create_vulkan_context:
+ * @self: a `GdkDisplay`
+ * @surface: (nullable): the `GdkSurface` to use or %NULL for a surfaceless
+ *   context
+ * @error: return location for an error
+ *
+ * Creates a new `GdkVulkanContext` for use with @display.
+ *
+ * If @surface is NULL, the context can not be used to draw to surfaces,
+ * it can only be used for custom rendering or compute.
+ *
+ * If the creation of the `GdkVulkanContext` failed, @error will be set.
+ *
+ * Returns: (transfer full): the newly created `GdkVulkanContext`, or
+ *   %NULL on error
+ */
+GdkVulkanContext *
+gdk_display_create_vulkan_context (GdkDisplay  *self,
+                                   GdkSurface  *surface,
+                                   GError     **error)
+{
+  g_return_val_if_fail (GDK_IS_DISPLAY (self), NULL);
+  g_return_val_if_fail (surface == NULL || GDK_IS_SURFACE (surface), NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (gdk_display_get_debug_flags (self) & GDK_DEBUG_VULKAN_DISABLE)
+    {
+      g_set_error_literal (error, GDK_VULKAN_ERROR, GDK_VULKAN_ERROR_NOT_AVAILABLE,
+                           _("Vulkan support disabled via GDK_DEBUG"));
+      return NULL;
+    }
+
+  if (GDK_DISPLAY_GET_CLASS (self)->vk_extension_name == NULL)
+    {
+      g_set_error (error, GDK_VULKAN_ERROR, GDK_VULKAN_ERROR_UNSUPPORTED,
+                   "The %s backend has no Vulkan support.", G_OBJECT_TYPE_NAME (self));
+      return FALSE;
+    }
+
+  if (surface)
+    {
+      return g_initable_new (GDK_DISPLAY_GET_CLASS (self)->vk_context_type,
+                             NULL,
+                             error,
+                             "surface", surface,
+                             NULL);
+    }
+  else
+    {
+      return g_initable_new (GDK_DISPLAY_GET_CLASS (self)->vk_context_type,
+                             NULL,
+                             error,
+                             "display", self,
+                             NULL);
+    }
+}
+
+gboolean
+gdk_display_has_vulkan_feature (GdkDisplay        *self,
+                                GdkVulkanFeatures  feature)
+{
+#ifdef GDK_RENDERING_VULKAN
+  return !!(self->vulkan_features & feature);
+#else
+  return FALSE;
+#endif
+}
+
 static void
 gdk_display_init_gl (GdkDisplay *self)
 {
@@ -1257,7 +1382,7 @@ gdk_display_init_gl (GdkDisplay *self)
       return;
     }
 
-  gdk_profiler_end_mark (before2, "realize OpenGL context", NULL);
+  gdk_profiler_end_mark (before2, "Realize OpenGL context", NULL);
 
   /* Only assign after realize, so GdkGLContext::realize() can use
    * gdk_display_get_gl_context() == NULL to differentiate between
@@ -1267,7 +1392,7 @@ gdk_display_init_gl (GdkDisplay *self)
 
   gdk_gl_backend_use (GDK_GL_CONTEXT_GET_CLASS (context)->backend_type);
 
-  gdk_profiler_end_mark (before, "initialize OpenGL", NULL);
+  gdk_profiler_end_mark (before, "Init OpenGL", NULL);
 }
 
 /**
@@ -1283,7 +1408,7 @@ gdk_display_init_gl (GdkDisplay *self)
  * Note that even if this function succeeds, creating a `GdkGLContext`
  * may still fail.
  *
- * This function is idempotent. Calling it multiple times will just 
+ * This function is idempotent. Calling it multiple times will just
  * return the same value or error.
  *
  * You never need to call this function, GDK will call it automatically
@@ -1377,7 +1502,6 @@ gdk_display_get_gl_context (GdkDisplay *self)
 }
 
 #ifdef HAVE_EGL
-#ifdef G_ENABLE_DEBUG
 static int
 strvcmp (gconstpointer p1,
          gconstpointer p2)
@@ -1414,6 +1538,7 @@ describe_egl_config (EGLDisplay egl_display,
                      EGLConfig  egl_config)
 {
   EGLint red, green, blue, alpha, type;
+  EGLint depth, stencil;
 
   if (egl_config == NULL)
     return g_strdup ("-");
@@ -1432,9 +1557,17 @@ describe_egl_config (EGLDisplay egl_display,
   else
     type = EGL_COLOR_COMPONENT_TYPE_FIXED_EXT;
 
-  return g_strdup_printf ("R%dG%dB%dA%d%s", red, green, blue, alpha, type == EGL_COLOR_COMPONENT_TYPE_FIXED_EXT ? "" : " float");
+  if (!eglGetConfigAttrib (egl_display, egl_config, EGL_DEPTH_SIZE, &depth))
+    depth = 0;
+
+  if (!eglGetConfigAttrib (egl_display, egl_config, EGL_STENCIL_SIZE, &stencil))
+    stencil = 0;
+
+  return g_strdup_printf ("R%dG%dB%dA%d%s, depth %d, stencil %d", red, green, blue, alpha,
+                          type == EGL_COLOR_COMPONENT_TYPE_FIXED_EXT ? "" : " float",
+                          depth,
+                          stencil);
 }
-#endif
 
 gpointer
 gdk_display_get_egl_config (GdkDisplay *self)
@@ -1638,6 +1771,23 @@ gdk_display_check_egl_extensions (EGLDisplay   egl_display,
   return TRUE;
 }
 
+static const char *
+find_egl_device (EGLDisplay egl_display)
+{
+  EGLAttrib value;
+  EGLDeviceEXT egl_device;
+
+  eglQueryDisplayAttribEXT (egl_display, EGL_DEVICE_EXT, &value);
+
+  egl_device = (EGLDeviceEXT)value;
+
+#ifndef EGL_DRM_RENDER_NODE_FILE_EXT
+#define EGL_DRM_RENDER_NODE_FILE_EXT 0x3377
+#endif
+
+  return eglQueryDeviceStringEXT (egl_device, EGL_DRM_RENDER_NODE_FILE_EXT);
+}
+
 gboolean
 gdk_display_init_egl (GdkDisplay  *self,
                       int          platform,
@@ -1647,7 +1797,6 @@ gdk_display_init_egl (GdkDisplay  *self,
 {
   GdkDisplayPrivate *priv = gdk_display_get_instance_private (self);
   G_GNUC_UNUSED gint64 start_time = GDK_PROFILER_CURRENT_TIME;
-  G_GNUC_UNUSED gint64 start_time2;
   int major, minor;
 
   if (!gdk_gl_backend_can_be_used (GDK_GL_EGL, error))
@@ -1674,7 +1823,6 @@ gdk_display_init_egl (GdkDisplay  *self,
       return FALSE;
     }
 
-  start_time2 = GDK_PROFILER_CURRENT_TIME;
   if (!eglInitialize (priv->egl_display, &major, &minor))
     {
       priv->egl_display = NULL;
@@ -1683,7 +1831,6 @@ gdk_display_init_egl (GdkDisplay  *self,
                            _("Could not initialize EGL display"));
       return FALSE;
     }
-  gdk_profiler_end_mark (start_time2, "eglInitialize", NULL);
 
   if (major < GDK_EGL_MIN_VERSION_MAJOR ||
       (major == GDK_EGL_MIN_VERSION_MAJOR && minor < GDK_EGL_MIN_VERSION_MINOR))
@@ -1723,6 +1870,10 @@ gdk_display_init_egl (GdkDisplay  *self,
     epoxy_has_egl_extension (priv->egl_display, "EGL_KHR_no_config_context");
   self->have_egl_pixel_format_float =
     epoxy_has_egl_extension (priv->egl_display, "EGL_EXT_pixel_format_float");
+  self->have_egl_dma_buf_import =
+    epoxy_has_egl_extension (priv->egl_display, "EGL_EXT_image_dma_buf_import_modifiers");
+  self->have_egl_dma_buf_export =
+    epoxy_has_egl_extension (priv->egl_display, "EGL_MESA_image_dma_buf_export");
 
   if (self->have_egl_no_config_context)
     priv->egl_config_high_depth = gdk_display_create_egl_config (self,
@@ -1731,15 +1882,22 @@ gdk_display_init_egl (GdkDisplay  *self,
   if (priv->egl_config_high_depth == NULL)
     priv->egl_config_high_depth = priv->egl_config;
 
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (self, OPENGL))
     {
       char *ext = describe_extensions (priv->egl_display);
       char *std_cfg = describe_egl_config (priv->egl_display, priv->egl_config);
       char *hd_cfg = describe_egl_config (priv->egl_display, priv->egl_config_high_depth);
+      const char *path;
+      struct stat buf = { .st_rdev = 0, };
+
+      path = find_egl_device (priv->egl_display);
+      if (path)
+        stat (path, &buf);
+
       gdk_debug_message ("EGL API version %d.%d found\n"
                          " - Vendor: %s\n"
                          " - Version: %s\n"
+                         " - Device: %s, %d %d\n"
                          " - Client APIs: %s\n"
                          " - Extensions:\n"
                          "\t%s\n"
@@ -1748,6 +1906,12 @@ gdk_display_init_egl (GdkDisplay  *self,
                          major, minor,
                          eglQueryString (priv->egl_display, EGL_VENDOR),
                          eglQueryString (priv->egl_display, EGL_VERSION),
+                         path ? path : "unknown",
+#ifdef HAVE_SYS_SYSMACROS_H
+                         major (buf.st_rdev), minor (buf.st_rdev),
+#else
+                         0, 0,
+#endif
                          eglQueryString (priv->egl_display, EGL_CLIENT_APIS),
                          ext, std_cfg,
                          priv->egl_config_high_depth == priv->egl_config ? "none" : hd_cfg);
@@ -1755,9 +1919,8 @@ gdk_display_init_egl (GdkDisplay  *self,
       g_free (std_cfg);
       g_free (ext);
     }
-#endif
 
-  gdk_profiler_end_mark (start_time, "init EGL", NULL);
+  gdk_profiler_end_mark (start_time, "Init EGL", NULL);
 
   return TRUE;
 }
@@ -1792,12 +1955,104 @@ gdk_display_get_egl_display (GdkDisplay *self)
 #endif
 }
 
+#ifdef HAVE_DMABUF
+static void
+gdk_display_add_dmabuf_downloader (GdkDisplay          *display,
+                                   GdkDmabufDownloader *downloader)
+{
+  gsize i;
+
+  if (downloader == NULL)
+    return;
+
+  /* dmabuf_downloaders is NULL-terminated */
+  for (i = 0; i < G_N_ELEMENTS (display->dmabuf_downloaders) - 1; i++)
+    {
+      if (display->dmabuf_downloaders[i] == NULL)
+        break;
+    }
+
+  g_assert (i < G_N_ELEMENTS (display->dmabuf_downloaders) - 1);
+
+  display->dmabuf_downloaders[i] = downloader;
+}
+#endif
+
+/* To support a drm format, we must be able to import it into GL
+ * using the relevant EGL extensions, and download it into a memory
+ * texture, possibly doing format conversion with shaders (in GSK).
+ */
+void
+gdk_display_init_dmabuf (GdkDisplay *self)
+{
+  GdkDmabufFormatsBuilder *builder;
+
+  if (self->dmabuf_formats != NULL)
+    return;
+
+  GDK_DISPLAY_DEBUG (self, DMABUF,
+                     "Beginning initialization of dmabuf support");
+
+  builder = gdk_dmabuf_formats_builder_new ();
+
+#ifdef HAVE_DMABUF
+  if (!GDK_DISPLAY_DEBUG_CHECK (self, DMABUF_DISABLE))
+    {
+#ifdef GDK_RENDERING_VULKAN
+      gdk_display_add_dmabuf_downloader (self, gdk_vulkan_get_dmabuf_downloader (self, builder));
+#endif
+
+#ifdef HAVE_EGL
+      gdk_display_add_dmabuf_downloader (self, gdk_dmabuf_get_egl_downloader (self, builder));
+#endif
+
+      gdk_dmabuf_formats_builder_add_formats (builder,
+                                              gdk_dmabuf_get_mmap_formats ());
+    }
+#endif
+
+  self->dmabuf_formats = gdk_dmabuf_formats_builder_free_to_formats (builder);
+
+  GDK_DISPLAY_DEBUG (self, DMABUF,
+                     "Initialized support for %zu dmabuf formats",
+                     gdk_dmabuf_formats_get_n_formats (self->dmabuf_formats));
+}
+
+/**
+ * gdk_display_get_dmabuf_formats:
+ * @display: a `GdkDisplay`
+ *
+ * Returns the dma-buf formats that are supported on this display.
+ *
+ * GTK may use OpenGL or Vulkan to support some formats.
+ * Calling this function will then initialize them if they aren't yet.
+ *
+ * The formats returned by this function can be used for negotiating
+ * buffer formats with producers such as v4l, pipewire or GStreamer.
+ *
+ * To learn more about dma-bufs, see [class@Gdk.DmabufTextureBuilder].
+ *
+ * Returns: (transfer none): a `GdkDmabufFormats` object
+ *
+ * Since: 4.14
+ */
+GdkDmabufFormats *
+gdk_display_get_dmabuf_formats (GdkDisplay *display)
+{
+  gdk_display_init_dmabuf (display);
+
+  return display->dmabuf_formats;
+}
+
 GdkDebugFlags
 gdk_display_get_debug_flags (GdkDisplay *display)
 {
+  if (display == NULL)
+    return _gdk_debug_flags;
+
   GdkDisplayPrivate *priv = gdk_display_get_instance_private (display);
 
-  return display ? priv->debug_flags : _gdk_debug_flags;
+  return priv->debug_flags;
 }
 
 void
@@ -1899,6 +2154,46 @@ gdk_display_set_rgba (GdkDisplay *display,
   g_object_notify_by_pspec (G_OBJECT (display), props[PROP_RGBA]);
 }
 
+/**
+ * gdk_display_supports_shadow_width: (attributes org.gtk.Method.get_property=shadow-width)
+ * @display: a `GdkDisplay`
+ *
+ * Returns whether it's possible for a surface to draw outside of the window area.
+ *
+ * If %TRUE is returned the application decides if it wants to draw shadows.
+ * If %FALSE is returned, the compositor decides if it wants to draw shadows.
+ *
+ * Returns: %TRUE if surfaces can draw shadows or
+ *   %FALSE if the display does not support this functionality.
+ *
+ * Since: 4.14
+ */
+gboolean
+gdk_display_supports_shadow_width (GdkDisplay *display)
+{
+  GdkDisplayPrivate *priv = gdk_display_get_instance_private (display);
+
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+
+  return priv->shadow_width;
+}
+
+void
+gdk_display_set_shadow_width (GdkDisplay *display,
+                              gboolean    shadow_width)
+{
+  GdkDisplayPrivate *priv = gdk_display_get_instance_private (display);
+
+  g_return_if_fail (GDK_IS_DISPLAY (display));
+
+  if (priv->shadow_width == shadow_width)
+    return;
+
+  priv->shadow_width = shadow_width;
+
+  g_object_notify_by_pspec (G_OBJECT (display), props[PROP_SHADOW_WIDTH]);
+}
+
 static void
 device_removed_cb (GdkSeat    *seat,
                    GdkDevice  *device,
