@@ -24,11 +24,16 @@
 #include "gdksnapshotprivate.h"
 #include "gdkprivate.h"
 
+#include <graphene.h>
+
 /* HACK: So we don't need to include any (not-yet-created) GSK or GTK headers */
+GdkSnapshot *   gtk_snapshot_new                        (void);
 void            gtk_snapshot_push_debug                 (GdkSnapshot            *snapshot,
                                                          const char             *message,
                                                          ...) G_GNUC_PRINTF (2, 3);
 void            gtk_snapshot_pop                        (GdkSnapshot            *snapshot);
+GdkPaintable *  gtk_snapshot_free_to_paintable          (GdkSnapshot            *snapshot,
+                                                         const graphene_size_t  *size);
 
 /**
  * GdkPaintable:
@@ -50,7 +55,7 @@ void            gtk_snapshot_pop                        (GdkSnapshot
  * to do, it is suggested that you scale your paintable ignoring any potential
  * aspect ratio.
  *
- * The contents that a `GdkPaintable` produces may depend on the [class@GdkSnapshot]
+ * The contents that a `GdkPaintable` produces may depend on the [class@Gdk.Snapshot]
  * passed to it. For example, paintables may decide to use more detailed images
  * on higher resolution screens or when OpenGL is available. A `GdkPaintable`
  * will however always produce the same output for the same snapshot.
@@ -58,7 +63,7 @@ void            gtk_snapshot_pop                        (GdkSnapshot
  * A `GdkPaintable` may change its contents, meaning that it will now produce
  * a different output with the same snapshot. Once that happens, it will call
  * [method@Gdk.Paintable.invalidate_contents] which will emit the
- * [signal@GdkPaintable::invalidate-contents] signal. If a paintable is known
+ * [signal@Gdk.Paintable::invalidate-contents] signal. If a paintable is known
  * to never change its contents, it will set the %GDK_PAINTABLE_STATIC_CONTENTS
  * flag. If a consumer cannot deal with changing contents, it may call
  * [method@Gdk.Paintable.get_current_image] which will return a static
@@ -69,7 +74,7 @@ void            gtk_snapshot_pop                        (GdkSnapshot
  * can use this information to layout thepaintable appropriately. Just like the
  * contents, the size of a paintable can change. A paintable will indicate this
  * by calling [method@Gdk.Paintable.invalidate_size] which will emit the
- * [signal@GdkPaintable::invalidate-size] signal. And just like for contents,
+ * [signal@Gdk.Paintable::invalidate-size] signal. And just like for contents,
  * if a paintable is known to never change its size, it will set the
  * %GDK_PAINTABLE_STATIC_SIZE flag.
  *
@@ -102,9 +107,21 @@ gdk_paintable_default_snapshot (GdkPaintable *paintable,
 static GdkPaintable *
 gdk_paintable_default_get_current_image (GdkPaintable *paintable)
 {
-  g_warning ("FIXME: implement by snapshotting at default size and returning a GskRendererNodePaintable");
+  int width, height;
+  GdkSnapshot *snapshot;
 
-  return paintable;
+  /* No need to check whether the paintable is static, as
+   * gdk_paintable_get_current_image () takes care of that already.  */
+
+  width = gdk_paintable_get_intrinsic_width (paintable);
+  height = gdk_paintable_get_intrinsic_height (paintable);
+
+  if (width <= 0 || height <= 0)
+    return gdk_paintable_new_empty (width, height);
+
+  snapshot = gtk_snapshot_new ();
+  gdk_paintable_snapshot (paintable, snapshot, width, height);
+  return gtk_snapshot_free_to_paintable (snapshot, NULL);
 }
 
 static GdkPaintableFlags
@@ -529,7 +546,7 @@ gdk_paintable_compute_concrete_size (GdkPaintable *paintable,
            * the missing dimension is calculated from the present
            * dimension and the intrinsic aspect ratio.
            * Otherwise, the missing dimension is taken from the default
-           * object size. 
+           * object size.
            */
           if (image_width)
             *concrete_width = image_width;
@@ -559,7 +576,7 @@ gdk_paintable_compute_concrete_size (GdkPaintable *paintable,
    * dimensions, the missing dimension is taken from the object's intrinsic
    * dimensions.
    * Otherwise, the missing dimension of the concrete object size is taken
-   * from the default object size. 
+   * from the default object size.
    */
   if (specified_width)
     {
@@ -664,7 +681,8 @@ gdk_empty_paintable_init (GdkEmptyPaintable *self)
  * This is often useful for implementing the
  * [vfunc@Gdk.Paintable.get_current_image] virtual function
  * when the paintable is in an incomplete state (like a
- * [class@Gtk.MediaStream] before receiving the first frame).
+ * [GtkMediaStream](../gtk4/class.MediaStream.html) before receiving
+ * the first frame).
  *
  * Returns: (transfer full): a `GdkPaintable`
  */
