@@ -28,6 +28,7 @@
 #include "gdkdisplay-win32.h"
 #include "gdkdevicemanager-win32.h"
 #include "gdkglcontext-win32.h"
+#include "gdksurface-win32.h"
 #include "gdkwin32display.h"
 #include "gdkwin32screen.h"
 #include "gdkwin32surface.h"
@@ -285,14 +286,16 @@ _gdk_win32_display_init_monitors (GdkWin32Display *win32_display)
       GdkMonitor *ex_monitor;
 
       w32_ex_monitor = GDK_WIN32_MONITOR (g_list_model_get_item (win32_display->monitors, i));
-      g_object_unref (w32_ex_monitor);
       ex_monitor = GDK_MONITOR (w32_ex_monitor);
 
-      if (!w32_ex_monitor->remove)
-        continue;
+      if (w32_ex_monitor->remove)
+        {
+          w32_ex_monitor->hmonitor = NULL;
+          g_list_store_remove (G_LIST_STORE (win32_display->monitors), i);
+          gdk_monitor_invalidate (ex_monitor);
+        }
 
-      g_list_store_remove (G_LIST_STORE (win32_display->monitors), i);
-      gdk_monitor_invalidate (ex_monitor);
+      g_object_unref (w32_ex_monitor);
     }
 
   for (i = 0; i < new_monitors->len; i++)
@@ -479,7 +482,7 @@ register_display_change_notification (GdkDisplay *display)
 
   wclass.lpszClassName = "GdkDisplayChange";
   wclass.lpfnWndProc = display_change_window_procedure;
-  wclass.hInstance = _gdk_app_hmodule;
+  wclass.hInstance = this_module ();
   wclass.style = CS_OWNDC;
 
   klass = RegisterClass (&wclass);
@@ -488,10 +491,10 @@ register_display_change_notification (GdkDisplay *display)
       display_win32->hwnd = CreateWindow (MAKEINTRESOURCE (klass),
                                           NULL, WS_POPUP,
                                           0, 0, 0, 0, NULL, NULL,
-                                          _gdk_app_hmodule, NULL);
+                                          this_module (), NULL);
       if (!display_win32->hwnd)
         {
-          UnregisterClass (MAKEINTRESOURCE (klass), _gdk_app_hmodule);
+          UnregisterClass (MAKEINTRESOURCE (klass), this_module ());
         }
     }
 }
@@ -545,6 +548,9 @@ _gdk_win32_display_open (const char *display_name)
 
   g_signal_emit_by_name (_gdk_display, "opened");
 
+  /* Precalculate keymap, see #6203 */
+  (void) _gdk_win32_display_get_keymap (_gdk_display);
+
   GDK_NOTE (MISC, g_print ("... _gdk_display now set up\n"));
 
   return _gdk_display;
@@ -645,15 +651,15 @@ gdk_win32_display_dispose (GObject *object)
 {
   GdkWin32Display *display_win32 = GDK_WIN32_DISPLAY (object);
 
+  if (display_win32->dummy_context_wgl.hglrc != NULL)
+    {
+      wglMakeCurrent (NULL, NULL);
+      wglDeleteContext (display_win32->dummy_context_wgl.hglrc);
+      display_win32->dummy_context_wgl.hglrc = NULL;
+    }
+
   if (display_win32->hwnd != NULL)
     {
-      if (display_win32->dummy_context_wgl.hglrc != NULL)
-        {
-          wglMakeCurrent (NULL, NULL);
-          wglDeleteContext (display_win32->dummy_context_wgl.hglrc);
-          display_win32->dummy_context_wgl.hglrc = NULL;
-        }
-
       DestroyWindow (display_win32->hwnd);
       display_win32->hwnd = NULL;
     }
@@ -1058,6 +1064,7 @@ gdk_win32_display_check_composited (GdkWin32Display *display)
     }
 
   gdk_display_set_composited (GDK_DISPLAY (display), composited);
+  gdk_display_set_shadow_width (GDK_DISPLAY (display), composited);
 }
 
 static void
@@ -1128,7 +1135,6 @@ gdk_win32_display_get_monitor_scale_factor (GdkWin32Display *display_win32,
         {
           if (GDK_WIN32_SURFACE (surface)->hdc == NULL)
             GDK_WIN32_SURFACE (surface)->hdc = GetDC (GDK_SURFACE_HWND (surface));
-
           hdc = GDK_WIN32_SURFACE (surface)->hdc;
         }
       else
@@ -1168,6 +1174,9 @@ gdk_win32_display_get_setting (GdkDisplay  *display,
                                const char *name,
                                GValue      *value)
 {
+  if (gdk_display_get_debug_flags (display) & GDK_DEBUG_DEFAULT_SETTINGS)
+    return FALSE;
+
   return _gdk_win32_get_setting (name, value);
 }
 
@@ -1181,11 +1190,7 @@ gdk_win32_display_init_gl (GdkDisplay  *display,
 {
   GdkWin32Display *display_win32 = GDK_WIN32_DISPLAY (display);
   HDC init_gl_hdc = NULL;
-
-  if (display_win32->dummy_context_wgl.hdc == NULL)
-    display_win32->dummy_context_wgl.hdc = GetDC (display_win32->hwnd);
-
-  init_gl_hdc = display_win32->dummy_context_wgl.hdc;
+  GdkGLContext *context;
 
   /*
    * No env vars set, do the regular GL initialization, first WGL and then EGL,
@@ -1197,8 +1202,10 @@ gdk_win32_display_init_gl (GdkDisplay  *display,
    * Disable defaulting to EGL as EGL is used more as a compatibility layer
    * on Windows rather than being a native citizen on Windows
    */
-  if (gdk_display_get_debug_flags (display) & (GDK_DEBUG_GL_EGL|GDK_DEBUG_GL_GLES))
+  if (gdk_display_get_debug_flags (display) & (GDK_DEBUG_GL_EGL|GDK_DEBUG_GL_DISABLE_GL))
     {
+      init_gl_hdc = GetDC (display_win32->hwnd);
+
       if (gdk_display_init_egl (display,
                                 EGL_PLATFORM_ANGLE_ANGLE,
                                 init_gl_hdc,
@@ -1207,6 +1214,7 @@ gdk_win32_display_init_gl (GdkDisplay  *display,
         {
           return g_object_new (GDK_TYPE_WIN32_GL_CONTEXT_EGL,
                                "display", display,
+                               "allowed-apis", GDK_GL_API_GLES,
                                NULL);
         }
       else
@@ -1214,16 +1222,15 @@ gdk_win32_display_init_gl (GdkDisplay  *display,
     }
 #endif
 
-  if (gdk_win32_display_init_wgl (display, error))
-    {
-      return g_object_new (GDK_TYPE_WIN32_GL_CONTEXT_WGL,
-                           "display", display,
-                           NULL);
-    }
+  context = gdk_win32_display_init_wgl (display, error);
+
+  if (context)
+    return context;
 
 #ifdef HAVE_EGL
   g_clear_error (error);
 
+  init_gl_hdc = GetDC (display_win32->hwnd);
   if (gdk_display_init_egl (display,
                             EGL_PLATFORM_ANGLE_ANGLE,
                             init_gl_hdc,
@@ -1267,6 +1274,8 @@ gdk_win32_display_class_init (GdkWin32DisplayClass *klass)
   object_class->dispose = gdk_win32_display_dispose;
   object_class->finalize = gdk_win32_display_finalize;
 
+  display_class->toplevel_type = GDK_TYPE_WIN32_TOPLEVEL;
+  display_class->popup_type = GDK_TYPE_WIN32_POPUP;
   display_class->cairo_context_type = GDK_TYPE_WIN32_CAIRO_CONTEXT;
 
   display_class->get_name = gdk_win32_display_get_name;
@@ -1280,7 +1289,6 @@ gdk_win32_display_class_init (GdkWin32DisplayClass *klass)
 
   display_class->get_next_serial = gdk_win32_display_get_next_serial;
   display_class->notify_startup_complete = gdk_win32_display_notify_startup_complete;
-  display_class->create_surface = _gdk_win32_display_create_surface;
 
   display_class->get_keymap = _gdk_win32_display_get_keymap;
 
