@@ -271,9 +271,10 @@ winpointer_make_event (GdkDeviceWinpointer *device,
   y /= impl->surface_scale;
 
   state = 0;
-  if (info->dwKeyStates & POINTER_MOD_CTRL)
+  /* Note that info->dwKeyStates is not reliable, use GetKeyState() */
+  if (GetKeyState (VK_CONTROL) < 0)
     state |= GDK_CONTROL_MASK;
-  if (info->dwKeyStates & POINTER_MOD_SHIFT)
+  if (GetKeyState (VK_SHIFT) < 0)
     state |= GDK_SHIFT_MASK;
   if (GetKeyState (VK_MENU) < 0)
     state |= GDK_ALT_MASK;
@@ -1003,7 +1004,7 @@ winpointer_notif_window_create (void)
   wndclassex.cbSize = sizeof (wndclassex);
   wndclassex.lpszClassName = L"GdkWin32WinpointerNotificationsWindowClass";
   wndclassex.lpfnWndProc = winpointer_notifications_window_procedure;
-  wndclassex.hInstance = _gdk_dll_hinstance;
+  wndclassex.hInstance = this_module ();
 
   if ((notifications_window_class = RegisterClassExW (&wndclassex)) == 0)
     {
@@ -1018,7 +1019,7 @@ winpointer_notif_window_create (void)
                                                        0, 0, 0, 0,
                                                        HWND_MESSAGE,
                                                        NULL,
-                                                       _gdk_dll_hinstance,
+                                                       this_module (),
                                                        NULL)))
     {
       WIN32_API_FAILED ("CreateWindowExW");
