@@ -39,7 +39,6 @@
 
 #include <string.h>
 
-#ifdef G_ENABLE_DEBUG
 static const char notify_modes[][19] = {
   "NotifyNormal",
   "NotifyGrab",
@@ -57,7 +56,6 @@ static const char notify_details[][23] = {
   "NotifyPointerRoot",
   "NotifyDetailNone"
 };
-#endif
 
 #define HAS_FOCUS(toplevel)                           \
   ((toplevel)->has_focus || (toplevel)->has_pointer_focus)
@@ -226,7 +224,6 @@ translate_valuator_class (GdkDisplay          *display,
     }
 
   _gdk_device_add_axis (device, use, min, max, resolution);
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (display, INPUT))
     {
       const char *label;
@@ -238,7 +235,6 @@ translate_valuator_class (GdkDisplay          *display,
 
       gdk_debug_message ("\n\taxis: %s %s", label, use == GDK_AXIS_IGNORE ? "(ignored)" : "(used)");
     }
-#endif
 }
 
 static void
@@ -521,7 +517,6 @@ create_device (GdkX11DeviceManagerXI2 *device_manager,
       break;
     }
 
-#ifdef G_ENABLE_DEBUG
   if (GDK_DISPLAY_DEBUG_CHECK (display, INPUT))
     {
       const char *type_names[] = { "logical", "physical", "floating" };
@@ -533,7 +528,6 @@ create_device (GdkX11DeviceManagerXI2 *device_manager,
                          dev->use == XIMasterPointer,
                          num_touches);
     }
-#endif
 
   if (dev->use != XIMasterKeyboard &&
       dev->use != XIMasterPointer)
@@ -1450,7 +1444,9 @@ _gdk_device_manager_xi2_handle_focus (GdkSurface *surface,
       GdkEvent *event;
 
       event = gdk_focus_event_new (surface, device, focus_in);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gdk_display_put_event (gdk_surface_get_display (surface), event);
+G_GNUC_END_IGNORE_DEPRECATIONS
       gdk_event_unref (event);
     }
 }
@@ -1917,7 +1913,6 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
         GdkTouchpadGesturePhase phase;
         double x, y;
 
-#ifdef G_ENABLE_DEBUG
         const char *event_name = "";
         switch (xev->evtype)
           {
@@ -1933,7 +1928,6 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
           default:
             break;
           }
-#endif
 
         GDK_DEBUG (EVENTS, "pinch gesture %s:\twindow %ld\n\tfinger_count: %u%s",
                            event_name,
@@ -1977,7 +1971,6 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
         GdkTouchpadGesturePhase phase;
         double x, y;
 
-#ifdef G_ENABLE_DEBUG
         const char *event_name = "";
         switch (xev->evtype)
           {
@@ -1993,7 +1986,6 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
           default:
             break;
           }
-#endif
 
         GDK_DEBUG (EVENTS, "swipe gesture %s:\twindow %ld\n\tfinger_count: %u%s",
                            event_name,
@@ -2064,19 +2056,10 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
             xev->detail != XINotifyInferior && xev->mode != XINotifyPassiveUngrab &&
             GDK_IS_TOPLEVEL (surface))
           {
-            if (gdk_x11_device_xi2_get_device_type ((GdkX11DeviceXI2 *) device) != GDK_X11_DEVICE_TYPE_LOGICAL)
-              _gdk_device_xi2_reset_scroll_valuators (GDK_X11_DEVICE_XI2 (source_device));
-            else
-              {
-                GList *physical_devices, *l;
+            GList *l;
 
-                physical_devices = gdk_device_list_physical_devices (source_device);
-
-                for (l = physical_devices; l; l = l->next)
-                  _gdk_device_xi2_reset_scroll_valuators (GDK_X11_DEVICE_XI2 (l->data));
-
-                g_list_free (physical_devices);
-              }
+            for (l = device_manager->devices; l; l = l->next)
+              _gdk_device_xi2_reset_scroll_valuators (GDK_X11_DEVICE_XI2 (l->data));
           }
 
         event = gdk_crossing_event_new (ev->evtype == XI_Enter
