@@ -162,7 +162,7 @@ gdk_x11_gl_context_glx_end_frame (GdkDrawContext *draw_context,
       if (display_x11->has_glx_video_sync)
         glXGetVideoSyncSGI (&end_frame_counter);
 
-      if (self->do_frame_sync && !display_x11->has_glx_swap_interval)
+      if (self->do_frame_sync && !display_x11->has_glx_sgi_swap_control && !display_x11->has_glx_swap_control)
         {
           glFinish ();
 
@@ -249,7 +249,7 @@ gdk_x11_gl_context_glx_make_current (GdkGLContext *context,
   if (!glXMakeContextCurrent (dpy, drawable, drawable, self->glx_context))
     return FALSE;
 
-  if (!surfaceless && GDK_X11_DISPLAY (display)->has_glx_swap_interval)
+  if (!surfaceless)
     {
       /* If the WM is compositing there is no particular need to delay
        * the swap when drawing on the offscreen, rendering to the screen
@@ -257,14 +257,35 @@ gdk_x11_gl_context_glx_make_current (GdkGLContext *context,
        * to the vblank. */
       do_frame_sync = ! gdk_display_is_composited (display);
 
-      if (do_frame_sync != self->do_frame_sync)
+      if (GDK_X11_DISPLAY (display)->has_glx_swap_control)
         {
-          self->do_frame_sync = do_frame_sync;
+          if (do_frame_sync != self->do_frame_sync)
+            {
+              self->do_frame_sync = do_frame_sync;
 
-          if (do_frame_sync)
-            glXSwapIntervalSGI (1);
-          else
-            glXSwapIntervalSGI (0);
+              if (do_frame_sync)
+                glXSwapIntervalEXT (dpy, drawable, 1);
+              else
+                glXSwapIntervalEXT (dpy, drawable, 0);
+            }
+        }
+      else if (GDK_X11_DISPLAY (display)->has_glx_sgi_swap_control)
+        {
+          /* If the WM is compositing there is no particular need to delay
+           * the swap when drawing on the offscreen, rendering to the screen
+           * happens later anyway, and its up to the compositor to sync that
+           * to the vblank. */
+          do_frame_sync = ! gdk_display_is_composited (display);
+
+          if (do_frame_sync != self->do_frame_sync)
+            {
+              self->do_frame_sync = do_frame_sync;
+
+              if (do_frame_sync)
+                glXSwapIntervalSGI (1);
+              else
+                glXSwapIntervalSGI (0);
+            }
         }
     }
 
@@ -287,29 +308,23 @@ gdk_x11_gl_context_glx_get_damage (GdkGLContext *context)
       glXQueryDrawable (dpy, gdk_x11_gl_context_glx_get_drawable (self),
                         GLX_BACK_BUFFER_AGE_EXT, &buffer_age);
 
-      switch (buffer_age)
+      if (buffer_age > 0 && buffer_age <= GDK_GL_MAX_TRACKED_BUFFERS)
         {
-          case 1:
-            return cairo_region_create ();
-            break;
+          cairo_region_t *damage = cairo_region_create ();
+          int i;
 
-          case 2:
-            if (context->old_updated_area[0])
-              return cairo_region_copy (context->old_updated_area[0]);
-            break;
+          for (i = 0; i < buffer_age - 1; i++)
+            {
+              if (context->old_updated_area[i] == NULL)
+                {
+                  cairo_region_destroy (damage);
+                  return GDK_GL_CONTEXT_CLASS (gdk_x11_gl_context_glx_parent_class)->get_damage (context);
+                }
 
-          case 3:
-            if (context->old_updated_area[0] &&
-                context->old_updated_area[1])
-              {
-                cairo_region_t *damage = cairo_region_copy (context->old_updated_area[0]);
-                cairo_region_union (damage, context->old_updated_area[1]);
-                return damage;
-              }
-            break;
+              cairo_region_union (damage, context->old_updated_area[i]);
+            }
 
-          default:
-            ;
+          return damage;
         }
     }
 
@@ -477,8 +492,11 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
   GdkSurface *surface = gdk_gl_context_get_surface (context);
   GLXContext ctx;
   int context_attribs[N_GLX_ATTRS], i = 0, flags = 0;
-  int min_major, min_minor, major, minor;
+  gsize major_idx, minor_idx;
+  GdkGLVersion version;
+  const GdkGLVersion* supported_versions;
   gboolean debug_bit, compat_bit;
+  gsize j;
 
   if (!gdk_gl_context_is_api_allowed (context, api, NULL))
     return 0;
@@ -488,10 +506,7 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
 
   /* We will use the default version matching the context status
    * unless the user requested a version which makes sense */
-  gdk_gl_context_get_matching_version (api, legacy, 0,
-                                       &min_major, &min_minor);
-  gdk_gl_context_get_clipped_version (context, min_major, min_minor,
-                                      &major, &minor);
+  gdk_gl_context_get_matching_version (context, api, legacy, &version);
 
   debug_bit = gdk_gl_context_get_debug_enabled (context);
   compat_bit = gdk_gl_context_get_forward_compatible (context);
@@ -511,9 +526,9 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
     context_attribs[i++] = GLX_CONTEXT_ES2_PROFILE_BIT_EXT;
 
   context_attribs[i++] = GLX_CONTEXT_MAJOR_VERSION_ARB;
-  context_attribs[i++] = major;
+  major_idx = i++;
   context_attribs[i++] = GLX_CONTEXT_MINOR_VERSION_ARB;
-  context_attribs[i++] = minor;
+  minor_idx = i++;
   context_attribs[i++] = GLX_CONTEXT_FLAGS_ARB;
   context_attribs[i++] = flags;
 
@@ -522,7 +537,7 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
 
   GDK_DISPLAY_DEBUG (display, OPENGL,
                      "Creating GLX context version %d.%d (debug:%s, forward:%s, legacy:%s, es:%s)",
-                     major, minor,
+                     gdk_gl_version_get_major (&version), gdk_gl_version_get_minor (&version),
                      debug_bit ? "yes" : "no",
                      compat_bit ? "yes" : "no",
                      legacy ? "yes" : "no",
@@ -531,27 +546,38 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
   if (share != NULL)
     share_glx = GDK_X11_GL_CONTEXT_GLX (share);
 
-  gdk_x11_display_error_trap_push (display);
+  supported_versions = gdk_gl_versions_get_for_api (api);
+  for (j = 0; gdk_gl_version_greater_equal (&supported_versions[j], &version); j++)
+    {
+      context_attribs [major_idx] = gdk_gl_version_get_major (&supported_versions[j]);
+      context_attribs [minor_idx] = gdk_gl_version_get_minor (&supported_versions[j]);
 
-  /* If we don't have access to GLX_ARB_create_context_profile, then
-   * we have to fall back to the old GLX 1.3 API.
-   */
-  if (legacy && !display_x11->has_glx_create_context)
-    ctx = glXCreateNewContext (gdk_x11_display_get_xdisplay (display),
-                               display_x11->glx_config,
-                               GLX_RGBA_TYPE,
-                               share_glx != NULL ? share_glx->glx_context : NULL,
-                               TRUE);
+      /* If we don't have access to GLX_ARB_create_context_profile, then
+       * we have to fall back to the old GLX 1.3 API.
+       */
+      if (legacy && !display_x11->has_glx_create_context)
+        ctx = glXCreateNewContext (gdk_x11_display_get_xdisplay (display),
+                                   display_x11->glx_config,
+                                   GLX_RGBA_TYPE,
+                                   share_glx != NULL ? share_glx->glx_context : NULL,
+                                   TRUE);
 
-  else
-    ctx = glXCreateContextAttribsARB (gdk_x11_display_get_xdisplay (display),
-                                      display_x11->glx_config,
-                                      share_glx != NULL ? share_glx->glx_context : NULL,
-                                      True,
-                                      context_attribs);
+      else
+        ctx = glXCreateContextAttribsARB (gdk_x11_display_get_xdisplay (display),
+                                          display_x11->glx_config,
+                                          share_glx != NULL ? share_glx->glx_context : NULL,
+                                          True,
+                                          context_attribs);
 
-  if (gdk_x11_display_error_trap_pop (display) || ctx == NULL)
-    return 0;
+      if (ctx != NULL)
+        break;
+    }
+
+  if (ctx == NULL)
+    {
+      GDK_DISPLAY_DEBUG (display, OPENGL, "Failed to create a GLX context");
+      return 0;
+    }
 
   GDK_DISPLAY_DEBUG (display, OPENGL,
                      "Realized GLX context[%p], %s, version: %d.%d",
@@ -561,6 +587,7 @@ gdk_x11_context_create_glx_context (GdkGLContext *context,
                      display_x11->glx_version % 10);
 
   context_glx->glx_context = ctx;
+  gdk_gl_context_set_version (context, &supported_versions[j]);
   gdk_gl_context_set_is_legacy (context, legacy);
 
 #ifdef HAVE_XDAMAGE
@@ -622,9 +649,6 @@ gdk_x11_gl_context_glx_realize (GdkGLContext  *context,
   /* If there is no glXCreateContextAttribsARB() then we default to legacy */
   legacy = !GDK_X11_DISPLAY (display)->has_glx_create_context;
 
-  if (gdk_display_get_debug_flags (display) & GDK_DEBUG_GL_LEGACY)
-    legacy = TRUE;
-
   /* We cannot share legacy contexts with core profile ones, so the
    * shared context is the one that decides if we're going to create
    * a legacy context or not.
@@ -632,25 +656,42 @@ gdk_x11_gl_context_glx_realize (GdkGLContext  *context,
   if (share != NULL && gdk_gl_context_is_legacy (share))
     legacy = TRUE;
 
+  gdk_x11_display_error_trap_push (display);
+
+  /* Increase XNextRequest because GLX may fake errors with the last request
+   * and we want the error trap to catch them */
+  XChangeWindowAttributes (GDK_DISPLAY_XDISPLAY (display),
+                           GDK_X11_DISPLAY (display)->leader_window,
+                           0,
+                           (XSetWindowAttributes[1]) { 0, });
+
   if (preferred_api == GDK_GL_API_GL)
     {
-      if ((api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, legacy)) ||
-          (api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GLES, legacy)) ||
-          (api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, TRUE)))
-        return api;
+      api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, legacy);
+      if (api == 0)
+        api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GLES, legacy);
+      if (api == 0)
+        api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, TRUE);
     }
   else
     {
-      if ((api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GLES, FALSE)) ||
-          (api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, legacy)) ||
-          (api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, TRUE)))
-        return api;
+      api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GLES, FALSE);
+      if (api == 0)
+        api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, legacy);
+      if (api == 0)
+        api = gdk_x11_context_create_glx_context (context, GDK_GL_API_GL, TRUE);
     }
 
-  g_set_error_literal (error, GDK_GL_ERROR,
-                       GDK_GL_ERROR_NOT_AVAILABLE,
-                       _("Unable to create a GL context"));
-  return 0;
+  gdk_x11_display_error_trap_pop_ignored (display);
+
+  if (api == 0)
+    {
+      g_set_error_literal (error, GDK_GL_ERROR,
+                           GDK_GL_ERROR_NOT_AVAILABLE,
+                           _("Unable to create a GL context"));
+    }
+
+  return api;
 }
 
 #undef N_GLX_ATTRS
@@ -925,8 +966,10 @@ gdk_x11_display_init_glx (GdkX11Display  *display_x11,
     epoxy_has_glx_extension (dpy, screen_num, "GLX_ARB_create_context_profile");
   display_x11->has_glx_create_es2_context =
     epoxy_has_glx_extension (dpy, screen_num, "GLX_EXT_create_context_es2_profile");
-  display_x11->has_glx_swap_interval =
+  display_x11->has_glx_sgi_swap_control =
     epoxy_has_glx_extension (dpy, screen_num, "GLX_SGI_swap_control");
+  display_x11->has_glx_swap_control =
+    epoxy_has_glx_extension (dpy, screen_num, "GLX_EXT_swap_control");
   display_x11->has_glx_texture_from_pixmap =
     epoxy_has_glx_extension (dpy, screen_num, "GLX_EXT_texture_from_pixmap");
   display_x11->has_glx_video_sync =
@@ -987,6 +1030,7 @@ gdk_x11_display_init_glx (GdkX11Display  *display_x11,
                        "\t* GLX_ARB_create_context_profile: %s\n"
                        "\t* GLX_EXT_create_context_es2_profile: %s\n"
                        "\t* GLX_SGI_swap_control: %s\n"
+                       "\t* GLX_EXT_swap_control: %s\n"
                        "\t* GLX_EXT_texture_from_pixmap: %s\n"
                        "\t* GLX_SGI_video_sync: %s\n"
                        "\t* GLX_EXT_buffer_age: %s\n"
@@ -998,7 +1042,8 @@ gdk_x11_display_init_glx (GdkX11Display  *display_x11,
                      glXGetClientString (dpy, GLX_VENDOR),
                      display_x11->has_glx_create_context ? "yes" : "no",
                      display_x11->has_glx_create_es2_context ? "yes" : "no",
-                     display_x11->has_glx_swap_interval ? "yes" : "no",
+                     display_x11->has_glx_sgi_swap_control ? "yes" : "no",
+                     display_x11->has_glx_swap_control ? "yes" : "no",
                      display_x11->has_glx_texture_from_pixmap ? "yes" : "no",
                      display_x11->has_glx_video_sync ? "yes" : "no",
                      display_x11->has_glx_buffer_age ? "yes" : "no",
