@@ -35,6 +35,7 @@ G_DEFINE_TYPE (GskBroadwayRenderer, gsk_broadway_renderer, GSK_TYPE_RENDERER)
 
 static gboolean
 gsk_broadway_renderer_realize (GskRenderer  *renderer,
+                               GdkDisplay   *display,
                                GdkSurface   *surface,
                                GError      **error)
 {
@@ -250,6 +251,7 @@ collect_reused_child_nodes (GskRenderer *renderer,
       /* Leaf nodes */
 
     case GSK_TEXTURE_NODE:
+    case GSK_TEXTURE_SCALE_NODE:
     case GSK_CAIRO_NODE:
     case GSK_COLOR_NODE:
     case GSK_BORDER_NODE:
@@ -269,6 +271,10 @@ collect_reused_child_nodes (GskRenderer *renderer,
     case GSK_BLEND_NODE:
     case GSK_CROSS_FADE_NODE:
     case GSK_BLUR_NODE:
+    case GSK_MASK_NODE:
+    case GSK_FILL_NODE:
+    case GSK_STROKE_NODE:
+    case GSK_SUBSURFACE_NODE:
 
     default:
 
@@ -449,7 +455,7 @@ get_colorized_texture (GdkTexture *texture,
                        const graphene_matrix_t *color_matrix,
                        const graphene_vec4_t *color_offset)
 {
-  cairo_surface_t *surface = gdk_texture_download_surface (texture);
+  cairo_surface_t *surface;
   cairo_surface_t *image_surface;
   graphene_vec4_t pixel;
   guint32* pixel_data;
@@ -471,6 +477,7 @@ get_colorized_texture (GdkTexture *texture,
         return g_object_ref (colorized->texture);
     }
 
+  surface = gdk_texture_download_surface (texture);
   image_surface = cairo_surface_map_to_image (surface, NULL);
   data = cairo_image_surface_get_data (image_surface);
   width = cairo_image_surface_get_width (image_surface);
@@ -532,6 +539,8 @@ get_colorized_texture (GdkTexture *texture,
                               colorized_list, (GDestroyNotify)colorized_texture_free_list);
     }
 
+  cairo_surface_destroy (surface);
+
   return colorized_texture;
 }
 
@@ -607,7 +616,7 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
             }
 
           texture = gdk_texture_new_for_surface (image_surface);
-          g_ptr_array_add (self->node_textures, g_object_ref (texture)); /* Transfers ownership to node_textures */
+          g_ptr_array_add (self->node_textures, texture); /* Transfers ownership to node_textures */
           texture_id = gdk_broadway_display_ensure_texture (display, texture);
 
           add_rect (nodes, &node->bounds, offset_x, offset_y);
@@ -796,6 +805,11 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
         }
       return;
 
+    case GSK_SUBSURFACE_NODE:
+      gsk_broadway_renderer_add_node (renderer,
+                                      gsk_subsurface_node_get_child (node), offset_x, offset_y, clip_bounds);
+     return;
+
       /* Generic nodes */
 
     case GSK_CONTAINER_NODE:
@@ -810,7 +824,7 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
               /* We prune fully clipped children, but we only do this for container_node, as
                * we don't have a way for any other nodes to say there are children missing (i.e.
                * bins always assume there is a child).
-               * Pruning is really only useful for large sets of children anyway, so thats
+               * Pruning is really only useful for large sets of children anyway, so that's
                * probably fine. */
               GskRenderNode *child = gsk_container_node_get_child (node, i);
               if (node_is_visible (child, clip_bounds))
@@ -845,6 +859,8 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
       }
       break; /* Fallback */
 
+    case GSK_MASK_NODE:
+    case GSK_TEXTURE_SCALE_NODE:
     case GSK_TEXT_NODE:
     case GSK_RADIAL_GRADIENT_NODE:
     case GSK_REPEATING_LINEAR_GRADIENT_NODE:
@@ -855,6 +871,8 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
     case GSK_CROSS_FADE_NODE:
     case GSK_BLUR_NODE:
     case GSK_GL_SHADER_NODE:
+    case GSK_FILL_NODE:
+    case GSK_STROKE_NODE:
     default:
       break; /* Fallback */
     }
@@ -894,6 +912,8 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
       add_float (nodes, width);
       add_float (nodes, height);
       add_uint32 (nodes, texture_id);
+
+      cairo_surface_destroy (surface);
     }
 }
 
