@@ -24,17 +24,11 @@
 #include "gtkapplicationprivate.h"
 #include "gtkwidgetprivate.h"
 #include "gtkwindowprivate.h"
-#include "gtkheaderbar.h"
 #include "gtkpopovermenubar.h"
-#include "gtkintl.h"
 #include "gtksettings.h"
 #include "gtkshortcutswindowprivate.h"
 #include "gtktooltipprivate.h"
 
-#if defined(HAVE_GIO_UNIX) && !defined(__APPLE__)
-#include <gio/gdesktopappinfo.h>
-#endif
-
 /**
  * GtkApplicationWindow:
  *
@@ -685,10 +679,7 @@ gtk_application_window_class_init (GtkApplicationWindowClass *class)
    * of whether the desktop shell is showing it or not.
    */
   gtk_application_window_properties[PROP_SHOW_MENUBAR] =
-    g_param_spec_boolean ("show-menubar",
-                          P_("Show a menubar"),
-                          P_("TRUE if the window should show a "
-                             "menubar at the top of the window"),
+    g_param_spec_boolean ("show-menubar", NULL, NULL,
                           FALSE, G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
   g_object_class_install_properties (object_class, N_PROPS, gtk_application_window_properties);
 }
@@ -792,7 +783,7 @@ show_help_overlay (GSimpleAction *action,
   GtkApplicationWindowPrivate *priv = gtk_application_window_get_instance_private (window);
 
   if (priv->help_overlay)
-    gtk_widget_show (GTK_WIDGET (priv->help_overlay));
+    gtk_window_present (GTK_WINDOW (priv->help_overlay));
 }
 
 /**
