@@ -22,6 +22,8 @@
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkprivate.h"
 #include "gtkstylepropertyprivate.h"
+#include "gtkcssstyleprivate.h"
+#include "gtkstyleproviderprivate.h"
 
 #include "gdk/gdkhslaprivate.h"
 #include "gdk/gdkrgbaprivate.h"
@@ -88,7 +90,7 @@ gtk_css_value_color_free (GtkCssValue *color)
       break;
     }
 
-  g_slice_free (GtkCssValue, color);
+  g_free (color);
 }
 
 static GtkCssValue *
@@ -720,7 +722,7 @@ _gtk_css_color_value_parse (GtkCssParser *parser)
     {
       const GtkCssToken *token = gtk_css_parser_get_token (parser);
 
-      value = _gtk_css_color_value_new_name (token->string.string);
+      value = _gtk_css_color_value_new_name (gtk_css_token_get_string (token));
       gtk_css_parser_consume_token (parser);
 
       return value;
