@@ -17,8 +17,7 @@
  * Authors: Alexander Larsson <alexl@gnome.org>
  */
 
-#ifndef __GTK_CSS_INITIAL_VALUE_PRIVATE_H__
-#define __GTK_CSS_INITIAL_VALUE_PRIVATE_H__
+#pragma once
 
 #include "gtkcssvalueprivate.h"
 
@@ -34,4 +33,3 @@ GtkCssValue *   _gtk_css_initial_value_new_compute    (guint             propert
 
 G_END_DECLS
 
-#endif /* __GTK_CSS_INITIAL_VALUE_PRIVATE_H__ */
