@@ -79,13 +79,9 @@ gtk_css_value_number_free (GtkCssValue *number)
 
       for (guint i = 0; i < n_terms; i++)
         _gtk_css_value_unref (number->calc.terms[i]);
+    }
 
-      g_slice_free1 (gtk_css_value_calc_get_size (n_terms), number);
-    }
-  else
-    {
-      g_slice_free (GtkCssValue, number);
-    }
+  g_free (number);
 }
 
 static double
@@ -380,6 +376,9 @@ gtk_css_dimension_value_new (double     value,
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 2 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 3 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 4 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 5 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 6 }} },
+    { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 7 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 8 }} },
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 16 }} }, /* Icon size default */
     { &GTK_CSS_VALUE_NUMBER, 1, TRUE, TYPE_DIMENSION, {{ GTK_CSS_PX, 32 }} },
@@ -418,16 +417,18 @@ gtk_css_dimension_value_new (double     value,
           value == 1 ||
           value == 2 ||
           value == 3 ||
-          value == 4)
+          value == 4 ||
+          value == 5 ||
+          value == 6 ||
+          value == 7 ||
+          value == 8)
         return _gtk_css_value_ref (&px_singletons[(int) value]);
-      if (value == 8)
-        return _gtk_css_value_ref (&px_singletons[5]);
       if (value == 16)
-        return _gtk_css_value_ref (&px_singletons[6]);
+        return _gtk_css_value_ref (&px_singletons[9]);
       if (value == 32)
-        return _gtk_css_value_ref (&px_singletons[7]);
+        return _gtk_css_value_ref (&px_singletons[10]);
       if (value == 64)
-        return _gtk_css_value_ref (&px_singletons[8]);
+        return _gtk_css_value_ref (&px_singletons[11]);
 
       break;
 
