@@ -142,7 +142,7 @@ gtk_css_transform_init_identity (GtkCssTransform     *transform,
     case GTK_CSS_TRANSFORM_ROTATE:
       transform->rotate.x = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
       transform->rotate.y = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
-      transform->rotate.z = _gtk_css_number_value_new (0, GTK_CSS_NUMBER);
+      transform->rotate.z = _gtk_css_number_value_new (1, GTK_CSS_NUMBER);
       transform->rotate.angle = _gtk_css_number_value_new (0, GTK_CSS_DEG);
       break;
     case GTK_CSS_TRANSFORM_SCALE:
@@ -266,11 +266,9 @@ gtk_css_value_transform_free (GtkCssValue *value)
   guint i;
 
   for (i = 0; i < value->n_transforms; i++)
-    {
-      gtk_css_transform_clear (&value->transforms[i]);
-    }
+    gtk_css_transform_clear (&value->transforms[i]);
 
-  g_slice_free1 (sizeof (GtkCssValue) + sizeof (GtkCssTransform) * (value->n_transforms - 1), value);
+  g_free (value);
 }
 
 /* returns TRUE if dest == src */
