@@ -90,6 +90,10 @@
  * </object>
  * ```
  *
+ * If a `GtkDropDown` is created in this manner, or with
+ * [ctor@Gtk.DropDown.new_from_strings], for instance, the object returned from
+ * [method@Gtk.DropDown.get_selected_item] will be a [class@Gtk.StringObject].
+ *
  * To learn more about the list widget framework, see the
  * [overview](section-list-widget.html).
  *
@@ -754,6 +758,21 @@ selected_item_changed (GtkDropDown *self,
     gtk_widget_set_opacity (icon, 0.0);
 }
 
+static void
+root_changed (GtkWidget   *box,
+              GParamSpec  *pspec,
+              GtkDropDown *self)
+{
+  GtkWidget *icon;
+
+  icon = gtk_widget_get_last_child (box);
+
+  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == self->popup)
+    gtk_widget_set_visible (icon, TRUE);
+  else
+    gtk_widget_set_visible (icon, FALSE);
+}
+
 static void
 bind_item (GtkSignalListItemFactory *factory,
            GtkListItem              *list_item,
@@ -763,13 +782,11 @@ bind_item (GtkSignalListItemFactory *factory,
   gpointer item;
   GtkWidget *box;
   GtkWidget *label;
-  GtkWidget *icon;
   GValue value = G_VALUE_INIT;
 
   item = gtk_list_item_get_item (list_item);
   box = gtk_list_item_get_child (list_item);
   label = gtk_widget_get_first_child (box);
-  icon = gtk_widget_get_last_child (box);
 
   if (self->expression &&
       gtk_expression_evaluate (self->expression, item, &value))
@@ -789,17 +806,13 @@ bind_item (GtkSignalListItemFactory *factory,
       g_critical ("Either GtkDropDown:factory or GtkDropDown:expression must be set");
     }
 
-  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == self->popup)
-    {
-      gtk_widget_set_visible (icon, TRUE);
-      g_signal_connect (self, "notify::selected-item",
-                        G_CALLBACK (selected_item_changed), list_item);
-      selected_item_changed (self, NULL, list_item);
-    }
-  else
-    {
-      gtk_widget_set_visible (icon, FALSE);
-    }
+  g_signal_connect (self, "notify::selected-item",
+                    G_CALLBACK (selected_item_changed), list_item);
+  selected_item_changed (self, NULL, list_item);
+
+  g_signal_connect (box, "notify::root",
+                    G_CALLBACK (root_changed), self);
+  root_changed (box, NULL, self);
 }
 
 static void
@@ -808,8 +821,12 @@ unbind_item (GtkSignalListItemFactory *factory,
              gpointer                  data)
 {
   GtkDropDown *self = data;
+  GtkWidget *box;
+
+  box = gtk_list_item_get_child (list_item);
 
   g_signal_handlers_disconnect_by_func (self, selected_item_changed, list_item);
+  g_signal_handlers_disconnect_by_func (box, root_changed, self);
 }
 
 static void
