@@ -17,8 +17,7 @@
  *
  * Author(s): Carlos Garnacho <carlosg@gnome.org>
  */
-#ifndef __GTK_GESTURE_ZOOM_PRIVATE_H__
-#define __GTK_GESTURE_ZOOM_PRIVATE_H__
+#pragma once
 
 #include "gtkgestureprivate.h"
 #include "gtkgesturezoom.h"
@@ -38,4 +37,3 @@ struct _GtkGestureZoomClass
   gpointer padding[10];
 };
 
-#endif /* __GTK_GESTURE_ZOOM_PRIVATE_H__ */
