@@ -19,7 +19,7 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 #include "config.h"
@@ -27,13 +27,11 @@
 #include "gtkimageprivate.h"
 
 #include "gtkiconhelperprivate.h"
-#include "gtkicontheme.h"
-#include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtksnapshot.h"
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
-#include "gdkpixbufutilsprivate.h"
+#include "gdktextureutilsprivate.h"
 
 #include <math.h>
 #include <string.h>
@@ -129,6 +127,8 @@ static void gtk_image_get_property         (GObject      *object,
                                             guint         prop_id,
                                             GValue       *value,
                                             GParamSpec   *pspec);
+static void gtk_image_clear_internal       (GtkImage *self,
+                                            gboolean  notify);
 
 enum
 {
@@ -174,21 +174,17 @@ gtk_image_class_init (GtkImageClass *class)
    * The `GdkPaintable` to display.
    */
   image_props[PROP_PAINTABLE] =
-      g_param_spec_object ("paintable",
-                           P_("Paintable"),
-                           P_("A GdkPaintable to display"),
+      g_param_spec_object ("paintable", NULL, NULL,
                            GDK_TYPE_PAINTABLE,
                            GTK_PARAM_READWRITE);
 
   /**
    * GtkImage:file: (attributes org.gtk.Property.set=gtk_image_set_from_file)
    *
-   * The `GFile to display.
+   * A path to the file to display.
    */
   image_props[PROP_FILE] =
-      g_param_spec_string ("file",
-                           P_("Filename"),
-                           P_("Filename to load and display"),
+      g_param_spec_string ("file", NULL, NULL,
                            NULL,
                            GTK_PARAM_READWRITE);
 
@@ -198,9 +194,7 @@ gtk_image_class_init (GtkImageClass *class)
    * The symbolic size to display icons at.
    */
   image_props[PROP_ICON_SIZE] =
-      g_param_spec_enum ("icon-size",
-                         P_("Icon size"),
-                         P_("Symbolic size to use for icon set or named icon"),
+      g_param_spec_enum ("icon-size", NULL, NULL,
                          GTK_TYPE_ICON_SIZE,
                          GTK_ICON_SIZE_INHERIT,
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
@@ -215,9 +209,7 @@ gtk_image_class_init (GtkImageClass *class)
    * `GTK_IMAGE_ICON_NAME`.
    */
   image_props[PROP_PIXEL_SIZE] =
-      g_param_spec_int ("pixel-size",
-                        P_("Pixel size"),
-                        P_("Pixel size to use for named icon"),
+      g_param_spec_int ("pixel-size", NULL, NULL,
                         -1, G_MAXINT,
                         -1,
                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
@@ -230,9 +222,7 @@ gtk_image_class_init (GtkImageClass *class)
    * If the icon theme is changed, the image will be updated automatically.
    */
   image_props[PROP_ICON_NAME] =
-      g_param_spec_string ("icon-name",
-                           P_("Icon Name"),
-                           P_("The name of the icon from the icon theme"),
+      g_param_spec_string ("icon-name", NULL, NULL,
                            NULL,
                            GTK_PARAM_READWRITE);
 
@@ -245,9 +235,7 @@ gtk_image_class_init (GtkImageClass *class)
    * automatically.
    */
   image_props[PROP_GICON] =
-      g_param_spec_object ("gicon",
-                           P_("Icon"),
-                           P_("The GIcon being displayed"),
+      g_param_spec_object ("gicon", NULL, NULL,
                            G_TYPE_ICON,
                            GTK_PARAM_READWRITE);
 
@@ -257,9 +245,7 @@ gtk_image_class_init (GtkImageClass *class)
    * A path to a resource file to display.
    */
   image_props[PROP_RESOURCE] =
-      g_param_spec_string ("resource",
-                           P_("Resource"),
-                           P_("The resource path being displayed"),
+      g_param_spec_string ("resource", NULL, NULL,
                            NULL,
                            GTK_PARAM_READWRITE);
 
@@ -269,9 +255,7 @@ gtk_image_class_init (GtkImageClass *class)
    * The representation being used for image data.
    */
   image_props[PROP_STORAGE_TYPE] =
-      g_param_spec_enum ("storage-type",
-                         P_("Storage type"),
-                         P_("The representation being used for image data"),
+      g_param_spec_enum ("storage-type", NULL, NULL,
                          GTK_TYPE_IMAGE_TYPE,
                          GTK_IMAGE_EMPTY,
                          GTK_PARAM_READABLE);
@@ -286,9 +270,7 @@ gtk_image_class_init (GtkImageClass *class)
    * %GTK_IMAGE_ICON_NAME and %GTK_IMAGE_GICON.
    */
   image_props[PROP_USE_FALLBACK] =
-      g_param_spec_boolean ("use-fallback",
-                            P_("Use Fallback"),
-                            P_("Whether to use icon names fallback"),
+      g_param_spec_boolean ("use-fallback", NULL, NULL,
                             FALSE,
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
@@ -314,7 +296,7 @@ gtk_image_finalize (GObject *object)
 {
   GtkImage *image = GTK_IMAGE (object);
 
-  gtk_image_clear (image);
+  gtk_image_clear_internal (image, FALSE);
 
   g_clear_object (&image->icon_helper);
 
@@ -367,7 +349,7 @@ gtk_image_set_property (GObject      *object,
     }
 }
 
-static void 
+static void
 gtk_image_get_property (GObject     *object,
 			guint        prop_id,
 			GValue      *value,
@@ -493,6 +475,9 @@ gtk_image_new_from_resource (const char *resource_path)
  * want that, you should use [ctor@Gtk.Image.new_from_icon_name].
  *
  * Returns: a new `GtkImage`
+ *
+ * Deprecated: 4.12: Use [ctor@Gtk.Image.new_from_paintable] and
+ *   [ctor@Gdk.Texture.new_for_pixbuf] instead
  */
 GtkWidget*
 gtk_image_new_from_pixbuf (GdkPixbuf *pixbuf)
@@ -501,9 +486,11 @@ gtk_image_new_from_pixbuf (GdkPixbuf *pixbuf)
 
   image = g_object_new (GTK_TYPE_IMAGE, NULL);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_image_set_from_pixbuf (image, pixbuf);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
-  return GTK_WIDGET (image);  
+  return GTK_WIDGET (image);
 }
 
 /**
@@ -530,7 +517,7 @@ gtk_image_new_from_paintable (GdkPaintable *paintable)
 
   gtk_image_set_from_paintable (image, paintable);
 
-  return GTK_WIDGET (image);  
+  return GTK_WIDGET (image);
 }
 
 /**
@@ -729,6 +716,8 @@ gtk_image_set_from_resource (GtkImage   *image,
  * Note: This is a helper for [method@Gtk.Image.set_from_paintable],
  * and you can't get back the exact pixbuf once this is called,
  * only a paintable.
+ *
+ * Deprecated: 4.12: Use [method@Gtk.Image.set_from_paintable] instead
  */
 void
 gtk_image_set_from_pixbuf (GtkImage  *image,
@@ -773,6 +762,7 @@ gtk_image_set_from_icon_name  (GtkImage    *image,
     _gtk_icon_helper_set_icon_name (image->icon_helper, icon_name);
 
   g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_NAME]);
+  g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STORAGE_TYPE]);
 
   g_object_thaw_notify (G_OBJECT (image));
 }
@@ -806,7 +796,8 @@ gtk_image_set_from_gicon  (GtkImage       *image,
     }
 
   g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_GICON]);
-  
+  g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STORAGE_TYPE]);
+
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -868,7 +859,8 @@ gtk_image_set_from_paintable (GtkImage     *image,
     }
 
   g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_PAINTABLE]);
-  
+  g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STORAGE_TYPE]);
+
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -1032,7 +1024,7 @@ gtk_image_snapshot (GtkWidget   *widget,
 
       x = (width - ceil (w)) / 2;
 
-      baseline = gtk_widget_get_allocated_baseline (widget);
+      baseline = gtk_widget_get_baseline (widget);
       if (baseline == -1)
         y = floor(height - ceil (h)) / 2;
       else
@@ -1080,7 +1072,7 @@ gtk_image_set_from_definition (GtkImage           *image,
   g_return_if_fail (GTK_IS_IMAGE (image));
 
   g_object_freeze_notify (G_OBJECT (image));
-  
+
   gtk_image_clear (image);
 
   if (def != NULL)
@@ -1090,6 +1082,8 @@ gtk_image_set_from_definition (GtkImage           *image,
       gtk_image_notify_for_storage_type (image, gtk_image_definition_get_storage_type (def));
     }
 
+  g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STORAGE_TYPE]);
+
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -1099,6 +1093,60 @@ gtk_image_get_definition (GtkImage *image)
   return gtk_icon_helper_get_definition (image->icon_helper);
 }
 
+static void
+gtk_image_clear_internal (GtkImage *self,
+                          gboolean  notify)
+{
+  GtkImageType storage_type = gtk_image_get_storage_type (self);
+  GObject *gobject = G_OBJECT (self);
+
+  if (notify)
+    {
+      if (storage_type != GTK_IMAGE_EMPTY)
+        g_object_notify_by_pspec (gobject, image_props[PROP_STORAGE_TYPE]);
+
+      g_object_notify_by_pspec (gobject, image_props[PROP_ICON_SIZE]);
+
+      gtk_image_notify_for_storage_type (self, storage_type);
+    }
+
+  if (self->filename)
+    {
+      g_free (self->filename);
+      self->filename = NULL;
+
+      if (notify)
+        g_object_notify_by_pspec (gobject, image_props[PROP_FILE]);
+    }
+
+  if (self->resource_path)
+    {
+      g_free (self->resource_path);
+      self->resource_path = NULL;
+
+      if (notify)
+        g_object_notify_by_pspec (gobject, image_props[PROP_RESOURCE]);
+    }
+
+  if (storage_type == GTK_IMAGE_PAINTABLE)
+    {
+      GdkPaintable *paintable = _gtk_icon_helper_peek_paintable (self->icon_helper);
+      const guint flags = gdk_paintable_get_flags (paintable);
+
+      if ((flags & GDK_PAINTABLE_STATIC_CONTENTS) == 0)
+        g_signal_handlers_disconnect_by_func (paintable,
+                                              gtk_image_paintable_invalidate_contents,
+                                              self);
+
+      if ((flags & GDK_PAINTABLE_STATIC_SIZE) == 0)
+        g_signal_handlers_disconnect_by_func (paintable,
+                                              gtk_image_paintable_invalidate_size,
+                                              self);
+    }
+
+  _gtk_icon_helper_clear (self->icon_helper);
+}
+
 /**
  * gtk_image_clear:
  * @image: a `GtkImage`
@@ -1108,50 +1156,10 @@ gtk_image_get_definition (GtkImage *image)
 void
 gtk_image_clear (GtkImage *image)
 {
-  GtkImageType storage_type;
+  g_return_if_fail (GTK_IS_IMAGE (image));
 
   g_object_freeze_notify (G_OBJECT (image));
-  storage_type = gtk_image_get_storage_type (image);
-
-  if (storage_type != GTK_IMAGE_EMPTY)
-    g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_STORAGE_TYPE]);
-
-  g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_ICON_SIZE]);
-
-  gtk_image_notify_for_storage_type (image, storage_type);
-
-  if (image->filename)
-    {
-      g_free (image->filename);
-      image->filename = NULL;
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_FILE]);
-    }
-
-  if (image->resource_path)
-    {
-      g_free (image->resource_path);
-      image->resource_path = NULL;
-      g_object_notify_by_pspec (G_OBJECT (image), image_props[PROP_RESOURCE]);
-    }
-
-  if (storage_type == GTK_IMAGE_PAINTABLE)
-    {
-      GdkPaintable *paintable = _gtk_icon_helper_peek_paintable (image->icon_helper);
-      const guint flags = gdk_paintable_get_flags (paintable);
-
-      if ((flags & GDK_PAINTABLE_STATIC_CONTENTS) == 0)
-        g_signal_handlers_disconnect_by_func (paintable,
-                                              gtk_image_paintable_invalidate_contents,
-                                              image);
-
-      if ((flags & GDK_PAINTABLE_STATIC_SIZE) == 0)
-        g_signal_handlers_disconnect_by_func (paintable,
-                                              gtk_image_paintable_invalidate_size,
-                                              image);
-    }
-
-  _gtk_icon_helper_clear (image->icon_helper);
-
+  gtk_image_clear_internal (image, TRUE);
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -1172,10 +1180,8 @@ gtk_image_measure (GtkWidget      *widget,
   if (orientation == GTK_ORIENTATION_VERTICAL)
     {
       baseline_align = gtk_image_get_baseline_align (GTK_IMAGE (widget));
-      if (minimum_baseline)
-        *minimum_baseline = *minimum * baseline_align;
-      if (natural_baseline)
-        *natural_baseline = *natural * baseline_align;
+      *minimum_baseline = *minimum * baseline_align;
+      *natural_baseline = *natural * baseline_align;
     }
 }
 
