@@ -105,6 +105,8 @@ static gboolean gtk_im_multicontext_delete_surrounding_cb   (GtkIMContext      *
 							     int                offset,
 							     int                n_chars,
 							     GtkIMMulticontext *multicontext);
+static gboolean gtk_im_multicontext_activate_osk_with_event (GtkIMContext *context,
+                                                             GdkEvent     *event);
 
 static void propagate_purpose (GtkIMMulticontext *context);
 
@@ -128,6 +130,7 @@ gtk_im_multicontext_class_init (GtkIMMulticontextClass *class)
   im_context_class->set_use_preedit = gtk_im_multicontext_set_use_preedit;
   im_context_class->set_surrounding_with_selection = gtk_im_multicontext_set_surrounding_with_selection;
   im_context_class->get_surrounding_with_selection = gtk_im_multicontext_get_surrounding_with_selection;
+  im_context_class->activate_osk_with_event = gtk_im_multicontext_activate_osk_with_event;
 
   gobject_class->finalize = gtk_im_multicontext_finalize;
 }
@@ -513,6 +516,19 @@ gtk_im_multicontext_set_surrounding_with_selection (GtkIMContext *context,
     gtk_im_context_set_surrounding_with_selection (delegate, text, len, cursor_index, anchor_index);
 }
 
+static gboolean
+gtk_im_multicontext_activate_osk_with_event (GtkIMContext *context,
+                                             GdkEvent     *event)
+{
+  GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
+  GtkIMContext *delegate = gtk_im_multicontext_get_delegate (multicontext);
+
+  if (delegate)
+    return gtk_im_context_activate_osk (delegate, event);
+  else
+    return FALSE;
+}
+
 static void
 gtk_im_multicontext_preedit_start_cb   (GtkIMContext      *delegate,
 					GtkIMMulticontext *multicontext)
