@@ -64,7 +64,7 @@
  *   //   background-color: magenta;
  *   //   border-style: solid;
  *   //   border-color: black;
- *   //   border-style: 1px;
+ *   //   border-width: 1px;
  *   // }
  *
  *   gtk_level_bar_add_offset_value (bar, "my-offset", 0.60);
@@ -90,8 +90,8 @@
  * # GtkLevelBar as GtkBuildable
  *
  * The `GtkLevelBar` implementation of the `GtkBuildable` interface supports a
- * custom <offsets> element, which can contain any number of <offset> elements,
- * each of which must have name and value attributes.
+ * custom `<offsets>` element, which can contain any number of `<offset>` elements,
+ * each of which must have "name" and "value" attributes.
  *
  * # CSS nodes
  *
@@ -121,6 +121,7 @@
  */
 #include "config.h"
 
+#include "gtkaccessiblerange.h"
 #include "gtkbinlayout.h"
 #include "gtkbuildable.h"
 #include "gtkbuilderprivate.h"
@@ -195,6 +196,7 @@ static void gtk_level_bar_buildable_init (GtkBuildableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GtkLevelBar, gtk_level_bar, GTK_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE_RANGE, NULL)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                                                 gtk_level_bar_buildable_init))
 
@@ -202,7 +204,7 @@ static GtkLevelBarOffset *
 gtk_level_bar_offset_new (const char *name,
                           double       value)
 {
-  GtkLevelBarOffset *offset = g_slice_new0 (GtkLevelBarOffset);
+  GtkLevelBarOffset *offset = g_new0 (GtkLevelBarOffset, 1);
 
   offset->name = g_strdup (name);
   offset->value = value;
@@ -214,7 +216,7 @@ static void
 gtk_level_bar_offset_free (GtkLevelBarOffset *offset)
 {
   g_free (offset->name);
-  g_slice_free (GtkLevelBarOffset, offset);
+  g_free (offset);
 }
 
 static int
@@ -764,7 +766,7 @@ gtk_level_bar_buildable_custom_tag_start (GtkBuildable       *buildable,
   if (strcmp (tagname, "offsets") != 0)
     return FALSE;
 
-  data = g_slice_new0 (OffsetsParserData);
+  data = g_new0 (OffsetsParserData, 1);
   data->self = GTK_LEVEL_BAR (buildable);
   data->builder = builder;
   data->offsets = NULL;
@@ -803,7 +805,7 @@ gtk_level_bar_buildable_custom_finished (GtkBuildable *buildable,
     }
 
   g_list_free_full (data->offsets, (GDestroyNotify) gtk_level_bar_offset_free);
-  g_slice_free (OffsetsParserData, data);
+  g_free (data);
 }
 
 static void
@@ -1099,7 +1101,7 @@ gtk_level_bar_new_for_interval (double min_value,
  * gtk_level_bar_get_min_value: (attributes org.gtk.Method.get_property=min-value)
  * @self: a `GtkLevelBar`
  *
- * Returns the `min-value of the `GtkLevelBar`.
+ * Returns the `min-value` of the `GtkLevelBar`.
  *
  * Returns: a positive value
  */
@@ -1134,7 +1136,7 @@ gtk_level_bar_get_max_value (GtkLevelBar *self)
  * Returns the `value` of the `GtkLevelBar`.
  *
  * Returns: a value in the interval between
- *   [property@Gtk.LevelBar:min-value[ and [property@Gtk.LevelBar:max-value]
+ *   [property@Gtk.LevelBar:min-value] and [property@Gtk.LevelBar:max-value]
  */
 double
 gtk_level_bar_get_value (GtkLevelBar *self)
