@@ -26,6 +26,7 @@
 
 #include "gtkpaned.h"
 
+#include "gtkaccessiblerange.h"
 #include "gtkcssboxesprivate.h"
 #include "gtkeventcontrollermotion.h"
 #include "gtkgesturepan.h"
@@ -101,13 +102,13 @@
  * gtk_widget_set_size_request (hpaned, 200, -1);
  *
  * gtk_paned_set_start_child (GTK_PANED (hpaned), frame1);
- * gtk_paned_set_start_child_resize (GTK_PANED (hpaned), TRUE);
- * gtk_paned_set_start_child_shrink (GTK_PANED (hpaned), FALSE);
+ * gtk_paned_set_resize_start_child (GTK_PANED (hpaned), TRUE);
+ * gtk_paned_set_shrink_start_child (GTK_PANED (hpaned), FALSE);
  * gtk_widget_set_size_request (frame1, 50, -1);
  *
  * gtk_paned_set_end_child (GTK_PANED (hpaned), frame2);
- * gtk_paned_set_end_child_resize (GTK_PANED (hpaned), FALSE);
- * gtk_paned_set_end_child_shrink (GTK_PANED (hpaned), FALSE);
+ * gtk_paned_set_resize_end_child (GTK_PANED (hpaned), FALSE);
+ * gtk_paned_set_shrink_end_child (GTK_PANED (hpaned), FALSE);
  * gtk_widget_set_size_request (frame2, 50, -1);
  * ```
  */
@@ -246,8 +247,12 @@ static void     update_drag                     (GtkPaned         *paned,
 
 static void gtk_paned_buildable_iface_init (GtkBuildableIface *iface);
 
+static void gtk_paned_accessible_range_init (GtkAccessibleRangeInterface *iface);
+
 G_DEFINE_TYPE_WITH_CODE (GtkPaned, gtk_paned, GTK_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE_RANGE,
+                                                gtk_paned_accessible_range_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                                                 gtk_paned_buildable_iface_init))
 
@@ -563,6 +568,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   _gtk_marshal_BOOLEAN__BOOLEAN,
                   G_TYPE_BOOLEAN, 1,
                   G_TYPE_BOOLEAN);
+  g_signal_set_va_marshaller (signals[CYCLE_CHILD_FOCUS],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__BOOLEANv);
 
   /**
    * GtkPaned::toggle-handle-focus:
@@ -583,6 +591,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   NULL, NULL,
                   _gtk_marshal_BOOLEAN__VOID,
                   G_TYPE_BOOLEAN, 0);
+  g_signal_set_va_marshaller (signals[TOGGLE_HANDLE_FOCUS],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__VOIDv);
 
   /**
    * GtkPaned::move-handle:
@@ -602,6 +613,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   _gtk_marshal_BOOLEAN__ENUM,
                   G_TYPE_BOOLEAN, 1,
                   GTK_TYPE_SCROLL_TYPE);
+  g_signal_set_va_marshaller (signals[MOVE_HANDLE],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__ENUMv);
 
   /**
    * GtkPaned::cycle-handle-focus:
@@ -624,6 +638,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   _gtk_marshal_BOOLEAN__BOOLEAN,
                   G_TYPE_BOOLEAN, 1,
                   G_TYPE_BOOLEAN);
+  g_signal_set_va_marshaller (signals[CYCLE_HANDLE_FOCUS],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__BOOLEANv);
 
   /**
    * GtkPaned::accept-position:
@@ -645,6 +662,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   NULL, NULL,
                   _gtk_marshal_BOOLEAN__VOID,
                   G_TYPE_BOOLEAN, 0);
+  g_signal_set_va_marshaller (signals[ACCEPT_POSITION],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__VOIDv);
 
   /**
    * GtkPaned::cancel-position:
@@ -668,6 +688,9 @@ gtk_paned_class_init (GtkPanedClass *class)
                   NULL, NULL,
                   _gtk_marshal_BOOLEAN__VOID,
                   G_TYPE_BOOLEAN, 0);
+  g_signal_set_va_marshaller (signals[CANCEL_POSITION],
+                              G_TYPE_FROM_CLASS (object_class),
+                              _gtk_marshal_BOOLEAN__VOIDv);
 
   /* F6 and friends */
   gtk_widget_class_add_binding_signal (widget_class,
@@ -804,6 +827,20 @@ gtk_paned_buildable_iface_init (GtkBuildableIface *iface)
   iface->add_child = gtk_paned_buildable_add_child;
 }
 
+static gboolean
+accessible_range_set_current_value (GtkAccessibleRange *accessible_range,
+                                    double              value)
+{
+  gtk_paned_set_position (GTK_PANED (accessible_range), (int) value + 0.5);
+  return TRUE;
+}
+
+static void
+gtk_paned_accessible_range_init (GtkAccessibleRangeInterface *iface)
+{
+  iface->set_current_value = accessible_range_set_current_value;
+}
+
 static gboolean
 initiates_touch_drag (GtkPaned *paned,
                       double    start_x,
@@ -849,6 +886,14 @@ gesture_drag_begin_cb (GtkGestureDrag *gesture,
   GdkDevice *device;
   gboolean is_touch;
 
+  /* Only drag the handle when it's visible */
+  if (!gtk_widget_get_child_visible (paned->handle_widget))
+    {
+      gtk_gesture_set_state (GTK_GESTURE (gesture),
+                             GTK_EVENT_SEQUENCE_DENIED);
+      return;
+    }
+
   sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
   event = gtk_gesture_get_last_event (GTK_GESTURE (gesture), sequence);
   device = gdk_event_get_device (event);
@@ -1555,11 +1600,14 @@ gtk_paned_new (GtkOrientation orientation)
  * If @child is `NULL`, the existing child will be removed.
  */
 void
-gtk_paned_set_start_child (GtkPaned *paned,
+gtk_paned_set_start_child (GtkPaned  *paned,
                            GtkWidget *child)
 {
   g_return_if_fail (GTK_IS_PANED (paned));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || paned->start_child == child || gtk_widget_get_parent (child) == NULL);
+
+  if (paned->start_child == child)
+    return;
 
   g_clear_pointer (&paned->start_child, gtk_widget_unparent);
 
@@ -1672,11 +1720,14 @@ gtk_paned_get_shrink_start_child (GtkPaned *paned)
  * If @child is `NULL`, the existing child will be removed.
  */
 void
-gtk_paned_set_end_child (GtkPaned *paned,
+gtk_paned_set_end_child (GtkPaned  *paned,
                          GtkWidget *child)
 {
   g_return_if_fail (GTK_IS_PANED (paned));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || paned->end_child == child || gtk_widget_get_parent (child) == NULL);
+
+  if (paned->end_child == child)
+    return;
 
   g_clear_pointer (&paned->end_child, gtk_widget_unparent);
 
@@ -2478,7 +2529,7 @@ gtk_paned_toggle_handle_focus (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_set_wide_handle: (attributes org.gtk.Method.set_propery=wide-handle)
+ * gtk_paned_set_wide_handle: (attributes org.gtk.Method.set_property=wide-handle)
  * @paned: a `GtkPaned`
  * @wide: the new value for the [property@Gtk.Paned:wide-handle] property
  *
