@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_POPOVER_PRIVATE_H__
-#define __GTK_POPOVER_PRIVATE_H__
+#pragma once
 
 #include "gtkpopover.h"
 
@@ -28,4 +27,3 @@ void       gtk_popover_disable_auto_mnemonics (GtkPopover *popover);
 
 G_END_DECLS
 
-#endif /* __GTK_POPOVER_PRIVATE_H__ */
