@@ -38,10 +38,11 @@
 #include <glib/gi18n-lib.h>
 #include "gtkprivate.h"
 #include "gtkmarshalers.h"
-#include "gtkstylecontext.h"
 #include "gtkeventcontrollerkey.h"
+#include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
-
+#include "gtkcssnodeprivate.h"
+#include "gtkcsspositionvalueprivate.h"
 
 /**
  * GtkSearchEntry:
@@ -105,6 +106,8 @@ enum {
 enum {
   PROP_0,
   PROP_PLACEHOLDER_TEXT,
+  PROP_INPUT_PURPOSE,
+  PROP_INPUT_HINTS,
   PROP_ACTIVATES_DEFAULT,
   PROP_SEARCH_DELAY,
   NUM_PROPERTIES,
@@ -125,8 +128,9 @@ struct _GtkSearchEntry
 
   guint search_delay;
 
+  GtkWidget *search_icon;
   GtkWidget *entry;
-  GtkWidget *icon;
+  GtkWidget *clear_icon;
 
   guint delayed_changed_id;
   gboolean content_changed;
@@ -166,10 +170,9 @@ gtk_search_entry_finalize (GObject *object)
 
   gtk_editable_finish_delegate (GTK_EDITABLE (entry));
 
-  gtk_widget_unparent (gtk_widget_get_first_child (GTK_WIDGET (entry)));
-
+  g_clear_pointer (&entry->search_icon, gtk_widget_unparent);
   g_clear_pointer (&entry->entry, gtk_widget_unparent);
-  g_clear_pointer (&entry->icon, gtk_widget_unparent);
+  g_clear_pointer (&entry->clear_icon, gtk_widget_unparent);
 
   if (entry->delayed_changed_id > 0)
     g_source_remove (entry->delayed_changed_id);
@@ -216,6 +219,14 @@ gtk_search_entry_set_property (GObject      *object,
                                       -1);
       break;
 
+    case PROP_INPUT_PURPOSE:
+      gtk_search_entry_set_input_purpose (entry, g_value_get_enum (value));
+      break;
+
+    case PROP_INPUT_HINTS:
+      gtk_search_entry_set_input_hints (entry, g_value_get_flags (value));
+      break;
+
     case PROP_ACTIVATES_DEFAULT:
       if (gtk_text_get_activates_default (GTK_TEXT (entry->entry)) != g_value_get_boolean (value))
         {
@@ -250,6 +261,14 @@ gtk_search_entry_get_property (GObject    *object,
       g_value_set_string (value, gtk_text_get_placeholder_text (GTK_TEXT (entry->entry)));
       break;
 
+    case PROP_INPUT_PURPOSE:
+      g_value_set_enum (value, gtk_text_get_input_purpose (GTK_TEXT (entry->entry)));
+      break;
+
+    case PROP_INPUT_HINTS:
+      g_value_set_flags (value, gtk_text_get_input_hints (GTK_TEXT (entry->entry)));
+      break;
+
     case PROP_ACTIVATES_DEFAULT:
       g_value_set_boolean (value, gtk_text_get_activates_default (GTK_TEXT (entry->entry)));
       break;
@@ -282,6 +301,144 @@ gtk_search_entry_mnemonic_activate (GtkWidget *widget,
   return TRUE;
 }
 
+static int
+get_spacing (GtkWidget *widget)
+{
+  GtkCssNode *node = gtk_widget_get_css_node (widget);
+  GtkCssStyle *style = gtk_css_node_get_style (node);
+
+  return _gtk_css_position_value_get_x (style->size->border_spacing, 100);
+}
+
+static void
+gtk_search_entry_measure (GtkWidget      *widget,
+                          GtkOrientation  orientation,
+                          int             for_size,
+                          int             *minimum,
+                          int             *natural,
+                          int             *minimum_baseline,
+                          int             *natural_baseline)
+{
+  GtkSearchEntry *entry = GTK_SEARCH_ENTRY (widget);
+  int text_min, text_nat;
+  int icon_min, icon_nat;
+  int spacing;
+
+  spacing = get_spacing (widget);
+
+  gtk_widget_measure (entry->entry,
+                      orientation,
+                      for_size,
+                      &text_min, &text_nat,
+                      minimum_baseline, natural_baseline);
+
+  *minimum = text_min;
+  *natural = text_nat;
+
+  gtk_widget_measure (entry->search_icon,
+                      GTK_ORIENTATION_HORIZONTAL,
+                      -1,
+                      &icon_min, &icon_nat,
+                      NULL, NULL);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      *minimum += icon_min + spacing;
+      *natural += icon_nat + spacing;
+    }
+  else
+    {
+      *minimum = MAX (*minimum, icon_min);
+      *natural = MAX (*natural, icon_nat);
+    }
+
+  gtk_widget_measure (entry->clear_icon,
+                      GTK_ORIENTATION_HORIZONTAL,
+                      -1,
+                      &icon_min, &icon_nat,
+                      NULL, NULL);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    {
+      *minimum += icon_min + spacing;
+      *natural += icon_nat + spacing;
+    }
+  else
+    {
+      *minimum = MAX (*minimum, icon_min);
+      *natural = MAX (*natural, icon_nat);
+
+      if (G_LIKELY (*minimum_baseline >= 0))
+        *minimum_baseline += (*minimum - text_min) / 2;
+      if (G_LIKELY (*natural_baseline >= 0))
+        *natural_baseline += (*natural - text_nat) / 2;
+    }
+}
+
+static void
+gtk_search_entry_size_allocate (GtkWidget *widget,
+                                int        width,
+                                int        height,
+                                int        baseline)
+{
+  GtkSearchEntry *entry = GTK_SEARCH_ENTRY (widget);
+  gboolean is_rtl;
+  GtkAllocation text_alloc;
+  GtkAllocation icon_alloc;
+  int icon_width;
+  int spacing;
+
+  spacing = get_spacing (widget);
+
+  is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
+
+  text_alloc.x = 0;
+  text_alloc.y = 0;
+  text_alloc.width = width;
+  text_alloc.height = height;
+
+  if (gtk_widget_get_valign (widget) != GTK_ALIGN_BASELINE_FILL &&
+      gtk_widget_get_valign (widget) != GTK_ALIGN_BASELINE_CENTER)
+    baseline = -1;
+
+  gtk_widget_measure (entry->search_icon,
+                      GTK_ORIENTATION_HORIZONTAL,
+                      -1,
+                      NULL, &icon_width,
+                      NULL, NULL);
+
+  icon_alloc.x = is_rtl ? width - icon_width : 0;
+  icon_alloc.y = 0;
+  icon_alloc.width = icon_width;
+  icon_alloc.height = height;
+
+  gtk_widget_size_allocate (entry->search_icon, &icon_alloc, baseline);
+
+  text_alloc.width -= icon_width + spacing;
+  text_alloc.x += is_rtl ? 0 : icon_width + spacing;
+
+  if (gtk_widget_get_child_visible (entry->clear_icon))
+    {
+      gtk_widget_measure (entry->clear_icon,
+                          GTK_ORIENTATION_HORIZONTAL,
+                          -1,
+                          NULL, &icon_width,
+                          NULL, NULL);
+
+      icon_alloc.x = is_rtl ? 0 : width - icon_width;
+      icon_alloc.y = 0;
+      icon_alloc.width = icon_width;
+      icon_alloc.height = height;
+
+      gtk_widget_size_allocate (entry->clear_icon, &icon_alloc, baseline);
+
+      text_alloc.width -= icon_width + spacing;
+      text_alloc.x += is_rtl ? icon_width + spacing : 0;
+    }
+
+  gtk_widget_size_allocate (entry->entry, &text_alloc, baseline);
+}
+
 static void
 gtk_search_entry_class_init (GtkSearchEntryClass *klass)
 {
@@ -295,6 +452,8 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
   widget_class->grab_focus = gtk_search_entry_grab_focus;
   widget_class->focus = gtk_widget_focus_child;
   widget_class->mnemonic_activate = gtk_search_entry_mnemonic_activate;
+  widget_class->measure = gtk_search_entry_measure;
+  widget_class->size_allocate = gtk_search_entry_size_allocate;
 
   klass->stop_search = gtk_search_entry_stop_search;
 
@@ -309,6 +468,34 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
                            NULL,
                            GTK_PARAM_READWRITE);
 
+  /**
+   * GtkSearchEntry:input-purpose:
+   *
+   * The purpose for the `GtkSearchEntry` input used to alter the
+   * behaviour of input methods.
+   *
+   * Since: 4.14
+   */
+  props[PROP_INPUT_PURPOSE] =
+      g_param_spec_enum ("input-purpose", NULL, NULL,
+                         GTK_TYPE_INPUT_PURPOSE,
+                         GTK_INPUT_PURPOSE_FREE_FORM,
+                         GTK_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * GtkSearchEntry:input-hints:
+   *
+   * The hints about input for the `GtkSearchEntry` used to alter the
+   * behaviour of input methods.
+   *
+   * Since: 4.14
+   */
+  props[PROP_INPUT_HINTS] =
+      g_param_spec_flags ("input-hints", NULL, NULL,
+                          GTK_TYPE_INPUT_HINTS,
+                          GTK_INPUT_HINT_NONE,
+                          GTK_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * GtkSearchEntry:activates-default:
    *
@@ -464,7 +651,6 @@ gtk_search_entry_class_init (GtkSearchEntryClass *klass)
                                        "stop-search",
                                        NULL);
 
-  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, I_("entry"));
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_SEARCH_BOX);
 }
@@ -485,19 +671,7 @@ static gboolean
 gtk_search_entry_accessible_get_platform_state (GtkAccessible              *self,
                                                 GtkAccessiblePlatformState  state)
 {
-  GtkSearchEntry *entry = GTK_SEARCH_ENTRY (self);
-
-  switch (state)
-    {
-    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSABLE:
-      return gtk_widget_get_focusable (GTK_WIDGET (entry->entry));
-    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSED:
-      return gtk_widget_has_focus (GTK_WIDGET (entry->entry));
-    case GTK_ACCESSIBLE_PLATFORM_STATE_ACTIVE:
-      return FALSE;
-    default:
-      g_assert_not_reached ();
-    }
+  return gtk_editable_delegate_get_accessible_platform_state (GTK_EDITABLE (self), state);
 }
 
 static void
@@ -561,7 +735,8 @@ gtk_search_entry_changed (GtkEditable    *editable,
 
   if (str == NULL || *str == '\0')
     {
-      gtk_widget_set_child_visible (entry->icon, FALSE);
+      gtk_widget_set_child_visible (entry->clear_icon, FALSE);
+      gtk_widget_queue_allocate (GTK_WIDGET (entry));
 
       if (entry->delayed_changed_id > 0)
         {
@@ -572,7 +747,8 @@ gtk_search_entry_changed (GtkEditable    *editable,
     }
   else
     {
-      gtk_widget_set_child_visible (entry->icon, TRUE);
+      gtk_widget_set_child_visible (entry->clear_icon, TRUE);
+      gtk_widget_queue_allocate (GTK_WIDGET (entry));
 
       /* Queue up the timeout */
       reset_timeout (entry);
@@ -610,17 +786,16 @@ catchall_click_press (GtkGestureClick *gesture,
 static void
 gtk_search_entry_init (GtkSearchEntry *entry)
 {
-  GtkWidget *icon;
   GtkGesture *press, *catchall;
 
   entry->search_delay = 150;
 
   /* The search icon is purely presentational */
-  icon = g_object_new (GTK_TYPE_IMAGE,
-                       "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
-                       "icon-name", "system-search-symbolic",
-                       NULL);
-  gtk_widget_set_parent (icon, GTK_WIDGET (entry));
+  entry->search_icon = g_object_new (GTK_TYPE_IMAGE,
+                                     "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
+                                     "icon-name", "system-search-symbolic",
+                                     NULL);
+  gtk_widget_set_parent (entry->search_icon, GTK_WIDGET (entry));
 
   entry->entry = gtk_text_new ();
   gtk_widget_set_parent (entry->entry, GTK_WIDGET (entry));
@@ -632,18 +807,18 @@ gtk_search_entry_init (GtkSearchEntry *entry)
   g_signal_connect (entry->entry, "notify", G_CALLBACK (notify_cb), entry);
   g_signal_connect (entry->entry, "activate", G_CALLBACK (activate_cb), entry);
 
-  entry->icon = g_object_new (GTK_TYPE_IMAGE,
-                              "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
-                              "icon-name", "edit-clear-symbolic",
-                              NULL);
-  gtk_widget_set_tooltip_text (entry->icon, _("Clear entry"));
-  gtk_widget_set_parent (entry->icon, GTK_WIDGET (entry));
-  gtk_widget_set_child_visible (entry->icon, FALSE);
+  entry->clear_icon = g_object_new (GTK_TYPE_IMAGE,
+                                    "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
+                                    "icon-name", "edit-clear-symbolic",
+                                    NULL);
+  gtk_widget_set_tooltip_text (entry->clear_icon, _("Clear Entry"));
+  gtk_widget_set_parent (entry->clear_icon, GTK_WIDGET (entry));
+  gtk_widget_set_child_visible (entry->clear_icon, FALSE);
 
   press = gtk_gesture_click_new ();
   g_signal_connect (press, "pressed", G_CALLBACK (gtk_search_entry_icon_press), entry);
   g_signal_connect (press, "released", G_CALLBACK (gtk_search_entry_icon_release), entry);
-  gtk_widget_add_controller (entry->icon, GTK_EVENT_CONTROLLER (press));
+  gtk_widget_add_controller (entry->clear_icon, GTK_EVENT_CONTROLLER (press));
 
   catchall = gtk_gesture_click_new ();
   g_signal_connect (catchall, "pressed",
@@ -652,6 +827,10 @@ gtk_search_entry_init (GtkSearchEntry *entry)
                              GTK_EVENT_CONTROLLER (catchall));
 
   gtk_widget_add_css_class (GTK_WIDGET (entry), I_("search"));
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (entry),
+                                  GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, TRUE,
+                                  -1);
 }
 
 /**
@@ -845,6 +1024,124 @@ gtk_search_entry_get_key_controller (GtkSearchEntry *entry)
   return gtk_text_get_key_controller (GTK_TEXT (entry->entry));
 }
 
+/**
+ * gtk_search_entry_get_placeholder_text:
+ * @entry: a `GtkSearchEntry`
+ *
+ * Gets the placeholder text associated with @entry.
+ *
+ * Returns: (nullable): The placeholder text.
+ *
+ * Since: 4.10
+ */
+const char *
+gtk_search_entry_get_placeholder_text (GtkSearchEntry *entry)
+{
+  g_return_val_if_fail (GTK_IS_SEARCH_ENTRY (entry), NULL);
+
+  return gtk_text_get_placeholder_text (GTK_TEXT (entry->entry));
+}
+
+/**
+ * gtk_search_entry_set_placeholder_text:
+ * @entry: a `GtkSearchEntry`
+ * @text: (nullable): the text to set as a placeholder
+ *
+ * Sets the placeholder text associated with @entry.
+ *
+ * Since: 4.10
+ */
+void
+gtk_search_entry_set_placeholder_text (GtkSearchEntry *entry,
+                                       const char     *text)
+{
+  g_return_if_fail (GTK_IS_SEARCH_ENTRY (entry));
+
+  gtk_text_set_placeholder_text (GTK_TEXT (entry->entry), text);
+}
+
+/**
+ * gtk_search_entry_get_input_purpose:
+ * @entry: a `GtkSearchEntry`
+ *
+ * Gets the input purpose of @entry.
+ *
+ * Returns: The input hints
+ *
+ * Since: 4.14
+ */
+GtkInputPurpose
+gtk_search_entry_get_input_purpose (GtkSearchEntry *entry)
+{
+  g_return_val_if_fail (GTK_IS_SEARCH_ENTRY (entry), GTK_INPUT_PURPOSE_FREE_FORM);
+
+  return gtk_text_get_input_purpose (GTK_TEXT (entry->entry));
+}
+
+/**
+ * gtk_search_entry_set_input_purpose:
+ * @entry: a `GtkSearchEntry`
+ * @purpose: the new input purpose
+ *
+ * Sets the input purpose of @entry.
+ *
+ * Since: 4.14
+ */
+void
+gtk_search_entry_set_input_purpose (GtkSearchEntry  *entry,
+                                    GtkInputPurpose  purpose)
+{
+  g_return_if_fail (GTK_IS_SEARCH_ENTRY (entry));
+
+  if (purpose != gtk_search_entry_get_input_purpose (entry))
+    {
+      gtk_text_set_input_purpose (GTK_TEXT (entry->entry), purpose);
+
+      g_object_notify_by_pspec (G_OBJECT (entry), props[PROP_INPUT_PURPOSE]);
+    }
+}
+
+/**
+ * gtk_search_entry_get_input_hints:
+ * @entry: a `GtkSearchEntry`
+ *
+ * Gets the input purpose for @entry.
+ *
+ * Returns: The input hints
+ *
+ * Since: 4.14
+ */
+GtkInputHints
+gtk_search_entry_get_input_hints (GtkSearchEntry *entry)
+{
+  g_return_val_if_fail (GTK_IS_SEARCH_ENTRY (entry), GTK_INPUT_HINT_NONE);
+
+  return gtk_text_get_input_hints (GTK_TEXT (entry->entry));
+}
+
+/**
+ * gtk_search_entry_set_input_hints:
+ * @entry: a `GtkSearchEntry`
+ * @hints: the new input hints
+ *
+ * Sets the input hints for @entry.
+ *
+ * Since: 4.14
+ */
+void
+gtk_search_entry_set_input_hints (GtkSearchEntry *entry,
+                                  GtkInputHints   hints)
+{
+  g_return_if_fail (GTK_IS_SEARCH_ENTRY (entry));
+
+  if (hints != gtk_search_entry_get_input_hints (entry))
+    {
+      gtk_text_set_input_hints (GTK_TEXT (entry->entry), hints);
+
+      g_object_notify_by_pspec (G_OBJECT (entry), props[PROP_INPUT_HINTS]);
+    }
+}
+
 GtkText *
 gtk_search_entry_get_text_widget (GtkSearchEntry *entry)
 {
