@@ -32,13 +32,12 @@
 #include "gtkspinbuttonprivate.h"
 
 #include "gtkaccessibleprivate.h"
+#include "gtkaccessiblerange.h"
 #include "gtkadjustment.h"
-#include "gtkbox.h"
 #include "gtkbutton.h"
 #include "gtkbuttonprivate.h"
 #include "gtkeditable.h"
-#include "gtkcelleditable.h"
-#include "gtkimage.h"
+#include "deprecated/gtkcelleditable.h"
 #include "gtktext.h"
 #include "gtkeventcontrollerkey.h"
 #include "gtkeventcontrollerfocus.h"
@@ -46,7 +45,6 @@
 #include "gtkeventcontrollerscroll.h"
 #include "gtkgestureclick.h"
 #include "gtkgestureswipe.h"
-#include "gtkicontheme.h"
 #include "gtkmarshalers.h"
 #include "gtkorientable.h"
 #include "gtkprivate.h"
@@ -119,7 +117,7 @@
  *   button = gtk_spin_button_new (adjustment, 1.0, 0);
  *   gtk_window_set_child (GTK_WINDOW (window), button);
  *
- *   gtk_widget_show (window);
+ *   gtk_window_present (GTK_WINDOW (window));
  * }
  * ```
  *
@@ -150,7 +148,7 @@
  *   button = gtk_spin_button_new (adjustment, 0.001, 3);
  *   gtk_window_set_child (GTK_WINDOW (window), button);
  *
- *   gtk_widget_show (window);
+ *   gtk_window_present (GTK_WINDOW (window));
  * }
  * ```
  *
@@ -180,7 +178,7 @@
  * below the text node. The orientation of the spin button is reflected in
  * the .vertical or .horizontal style class on the main node.
  *
- * # Accessiblity
+ * # Accessibility
  *
  * `GtkSpinButton` uses the %GTK_ACCESSIBLE_ROLE_SPIN_BUTTON role.
  */
@@ -220,6 +218,8 @@ struct _GtkSpinButton
   guint          timer_calls   : 3;
   guint          wrap          : 1;
   guint          editing_canceled : 1;
+  guint          edited : 1;
+  guint          activates_default : 1;
 };
 
 struct _GtkSpinButtonClass
@@ -230,6 +230,7 @@ struct _GtkSpinButtonClass
                  double        *new_value);
   int (*output) (GtkSpinButton *spin_button);
   void (*value_changed) (GtkSpinButton *spin_button);
+  void (*activate)      (GtkSpinButton *spin_button);
 
   /* Action signals for keybindings, do not connect to these */
   void (*change_value) (GtkSpinButton *spin_button,
@@ -240,6 +241,7 @@ struct _GtkSpinButtonClass
 
 enum {
   PROP_0,
+  PROP_ACTIVATES_DEFAULT,
   PROP_ADJUSTMENT,
   PROP_CLIMB_RATE,
   PROP_DIGITS,
@@ -259,6 +261,7 @@ enum
   INPUT,
   OUTPUT,
   VALUE_CHANGED,
+  ACTIVATE,
   CHANGE_VALUE,
   WRAPPED,
   LAST_SIGNAL
@@ -284,12 +287,15 @@ static gboolean gtk_spin_button_stop_spinning  (GtkSpinButton      *spin);
 static void gtk_spin_button_value_changed  (GtkAdjustment      *adjustment,
                                             GtkSpinButton      *spin_button);
 
+static void gtk_spin_button_changed (GtkEditable   *editable,
+                                     GtkSpinButton *spin_button);
+static void gtk_spin_button_real_activate (GtkSpinButton *spin_button);
 static void gtk_spin_button_activate       (GtkText            *entry,
                                             gpointer            user_data);
 static void gtk_spin_button_unset_adjustment (GtkSpinButton *spin_button);
 static void gtk_spin_button_set_orientation (GtkSpinButton     *spin_button,
                                              GtkOrientation     orientation);
-static void gtk_spin_button_snap           (GtkSpinButton      *spin_button,
+static double gtk_spin_button_snap         (GtkSpinButton      *spin_button,
                                             double              val);
 static void gtk_spin_button_insert_text    (GtkEditable        *editable,
                                             const char         *new_text,
@@ -308,6 +314,8 @@ static void gtk_spin_button_update_width_chars (GtkSpinButton *spin_button);
 
 static void gtk_spin_button_accessible_init (GtkAccessibleInterface *iface);
 
+static void gtk_spin_button_accessible_range_init (GtkAccessibleRangeInterface *iface);
+
 static guint spinbutton_signals[LAST_SIGNAL] = {0};
 static GParamSpec *spinbutton_props[NUM_SPINBUTTON_PROPS] = {NULL, };
 
@@ -315,6 +323,8 @@ G_DEFINE_TYPE_WITH_CODE (GtkSpinButton, gtk_spin_button, GTK_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE,
                                                 gtk_spin_button_accessible_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE_RANGE,
+                                                gtk_spin_button_accessible_range_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE,
                                                 gtk_spin_button_editable_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_CELL_EDITABLE,
@@ -363,6 +373,21 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   class->input = NULL;
   class->output = NULL;
   class->change_value = gtk_spin_button_real_change_value;
+  class->activate = gtk_spin_button_real_activate;
+
+  /**
+   * GtkSpinButton:activates-default: (attributes org.gtk.Property.get=gtk_spin_button_get_activates_default org.gtk.Property.set=gtk_spin_button_set_activates_default)
+   *
+   * Whether to activate the default widget when the spin button is activated.
+   *
+   * See [signal@Gtk.SpinButton::activate] for what counts as activation.
+   *
+   * Since: 4.14
+   */
+  spinbutton_props[PROP_ACTIVATES_DEFAULT] =
+    g_param_spec_boolean ("activates-default", NULL, NULL,
+                          FALSE,
+                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkSpinButton:adjustment: (attributes org.gtk.Property.get=gtk_spin_button_get_adjustment org.gtk.Property.set=gtk_spin_button_set_adjustment)
@@ -372,7 +397,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   spinbutton_props[PROP_ADJUSTMENT] =
     g_param_spec_object ("adjustment", NULL, NULL,
                          GTK_TYPE_ADJUSTMENT,
-                         GTK_PARAM_READWRITE);
+                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkSpinButton:climb-rate: (attributes org.gtk.Property.get=gtk_spin_button_get_climb_rate org.gtk.Property.set=gtk_spin_button_set_climb_rate)
@@ -477,6 +502,9 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
                   _gtk_marshal_INT__POINTER,
                   G_TYPE_INT, 1,
                   G_TYPE_POINTER);
+  g_signal_set_va_marshaller (spinbutton_signals[INPUT],
+                              G_TYPE_FROM_CLASS (gobject_class),
+                              _gtk_marshal_INT__POINTERv);
 
   /**
    * GtkSpinButton::output:
@@ -490,12 +518,10 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
    * on_output (GtkSpinButton *spin,
    *            gpointer       data)
    * {
-   *    GtkAdjustment *adjustment;
    *    char *text;
    *    int value;
    *
-   *    adjustment = gtk_spin_button_get_adjustment (spin);
-   *    value = (int)gtk_adjustment_get_value (adjustment);
+   *    value = gtk_spin_button_get_value_as_int (spin);
    *    text = g_strdup_printf ("%02d", value);
    *    gtk_editable_set_text (GTK_EDITABLE (spin), text):
    *    g_free (text);
@@ -514,6 +540,9 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
                   _gtk_boolean_handled_accumulator, NULL,
                   _gtk_marshal_BOOLEAN__VOID,
                   G_TYPE_BOOLEAN, 0);
+  g_signal_set_va_marshaller (spinbutton_signals[OUTPUT],
+                              G_TYPE_FROM_CLASS (gobject_class),
+                              _gtk_marshal_BOOLEAN__VOIDv);
 
   /**
    * GtkSpinButton::value-changed:
@@ -532,6 +561,31 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
                   NULL,
                   G_TYPE_NONE, 0);
 
+  /**
+   * GtkSpinButton::activate:
+   * @self: The widget on which the signal is emitted
+   *
+   * Emitted when the spin button is activated.
+   *
+   * The keybindings for this signal are all forms of the <kbd>Enter</kbd> key.
+   *
+   * If the <kbd>Enter</kbd> key results in the value being committed to the
+   * spin button, then activation does not occur until <kbd>Enter</kbd> is
+   * pressed again.
+   *
+   * Since: 4.14
+   */
+  spinbutton_signals[ACTIVATE] =
+    g_signal_new (I_("activate"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (GtkSpinButtonClass, activate),
+                  NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE, 0);
+
+  gtk_widget_class_set_activate_signal (widget_class, spinbutton_signals[ACTIVATE]);
+
   /**
    * GtkSpinButton::wrapped:
    * @spin_button: the object on which the signal was emitted
@@ -609,29 +663,34 @@ static gboolean
 gtk_spin_button_accessible_get_platform_state (GtkAccessible              *self,
                                                GtkAccessiblePlatformState  state)
 {
-  GtkSpinButton *spin_button = GTK_SPIN_BUTTON (self);
-
-  switch (state)
-    {
-    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSABLE:
-      return gtk_widget_get_focusable (spin_button->entry);
-    case GTK_ACCESSIBLE_PLATFORM_STATE_FOCUSED:
-      return gtk_widget_has_focus (spin_button->entry);
-    case GTK_ACCESSIBLE_PLATFORM_STATE_ACTIVE:
-      return FALSE;
-    default:
-      g_assert_not_reached ();
-    }
+  return gtk_editable_delegate_get_accessible_platform_state (GTK_EDITABLE (self), state);
 }
 
 static void
 gtk_spin_button_accessible_init (GtkAccessibleInterface *iface)
 {
   GtkAccessibleInterface *parent_iface = g_type_interface_peek_parent (iface);
+
   iface->get_at_context = parent_iface->get_at_context;
   iface->get_platform_state = gtk_spin_button_accessible_get_platform_state;
 }
 
+static gboolean
+accessible_range_set_current_value (GtkAccessibleRange *accessible_range,
+                                    double              value)
+{
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (accessible_range), value);
+  return TRUE;
+}
+
+static void
+gtk_spin_button_accessible_range_init (GtkAccessibleRangeInterface *iface)
+{
+  iface->set_current_value = accessible_range_set_current_value;
+}
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static void
 gtk_cell_editable_spin_button_activated (GtkText *text, GtkSpinButton *spin)
 {
@@ -692,6 +751,8 @@ gtk_spin_button_cell_editable_init (GtkCellEditableIface *iface)
   iface->start_editing = gtk_spin_button_start_editing;
 }
 
+G_GNUC_END_IGNORE_DEPRECATIONS
+
 static void
 gtk_spin_button_set_property (GObject      *object,
                               guint         prop_id,
@@ -714,6 +775,9 @@ gtk_spin_button_set_property (GObject      *object,
     {
       GtkAdjustment *adjustment;
 
+    case PROP_ACTIVATES_DEFAULT:
+      gtk_spin_button_set_activates_default (spin_button, g_value_get_boolean (value));
+      break;
     case PROP_ADJUSTMENT:
       adjustment = GTK_ADJUSTMENT (g_value_get_object (value));
       gtk_spin_button_set_adjustment (spin_button, adjustment);
@@ -779,6 +843,9 @@ gtk_spin_button_get_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_ACTIVATES_DEFAULT:
+      g_value_set_boolean (value, spin_button->activates_default);
+      break;
     case PROP_ADJUSTMENT:
       g_value_set_object (value, spin_button->adjustment);
       break;
@@ -988,6 +1055,7 @@ gtk_spin_button_init (GtkSpinButton *spin_button)
   GtkEventController *controller;
   GtkGesture *gesture;
 
+  spin_button->activates_default = FALSE;
   spin_button->adjustment = NULL;
   spin_button->timer = 0;
   spin_button->climb_rate = 0.0;
@@ -999,6 +1067,7 @@ gtk_spin_button_init (GtkSpinButton *spin_button)
   spin_button->digits = 0;
   spin_button->numeric = FALSE;
   spin_button->wrap = FALSE;
+  spin_button->edited = FALSE;
   spin_button->snap_to_ticks = FALSE;
   spin_button->width_chars = -1;
 
@@ -1011,6 +1080,7 @@ gtk_spin_button_init (GtkSpinButton *spin_button)
   gtk_widget_set_hexpand (spin_button->entry, TRUE);
   gtk_widget_set_vexpand (spin_button->entry, TRUE);
   g_signal_connect (spin_button->entry, "activate", G_CALLBACK (gtk_spin_button_activate), spin_button);
+  g_signal_connect (spin_button->entry, "changed", G_CALLBACK (gtk_spin_button_changed), spin_button);
   gtk_widget_set_parent (spin_button->entry, GTK_WIDGET (spin_button));
 
   spin_button->down_button = g_object_new (GTK_TYPE_BUTTON,
@@ -1049,8 +1119,7 @@ gtk_spin_button_init (GtkSpinButton *spin_button)
   g_signal_connect (gesture, "released", G_CALLBACK (button_released_cb), spin_button);
   g_signal_connect (gesture, "cancel", G_CALLBACK (button_cancel_cb), spin_button);
   gtk_widget_add_controller (GTK_WIDGET (spin_button->up_button), GTK_EVENT_CONTROLLER (gesture));
-  gtk_gesture_group (gtk_button_get_gesture (GTK_BUTTON (spin_button->up_button)),
-		     gesture);
+  gtk_gesture_group (gtk_button_get_gesture (GTK_BUTTON (spin_button->up_button)), gesture);
 
   gtk_spin_button_set_adjustment (spin_button, NULL);
 
@@ -1066,7 +1135,7 @@ gtk_spin_button_init (GtkSpinButton *spin_button)
                              GTK_EVENT_CONTROLLER (gesture));
 
   controller = gtk_event_controller_scroll_new (GTK_EVENT_CONTROLLER_SCROLL_VERTICAL |
-				                GTK_EVENT_CONTROLLER_SCROLL_DISCRETE);
+                                                GTK_EVENT_CONTROLLER_SCROLL_DISCRETE);
   g_signal_connect (controller, "scroll",
                     G_CALLBACK (scroll_controller_scroll), spin_button);
   gtk_widget_add_controller (GTK_WIDGET (spin_button), controller);
@@ -1205,11 +1274,13 @@ gtk_spin_button_set_orientation (GtkSpinButton  *spin,
     {
       /* Current orientation of the box is vertical! */
       gtk_widget_insert_after (spin->up_button, GTK_WIDGET (spin), spin->down_button);
+      gtk_box_layout_set_baseline_child (layout_manager, 0);
     }
   else
     {
       /* Current orientation of the box is horizontal! */
       gtk_widget_insert_before (spin->up_button, GTK_WIDGET (spin), spin->entry);
+      gtk_box_layout_set_baseline_child (layout_manager, 1);
     }
 
   g_object_notify (G_OBJECT (spin), "orientation");
@@ -1345,6 +1416,8 @@ gtk_spin_button_value_changed (GtkAdjustment *adjustment,
   update_buttons_sensitivity (spin_button);
 
   g_object_notify_by_pspec (G_OBJECT (spin_button), spinbutton_props[PROP_VALUE]);
+
+  spin_button->edited = FALSE;
 }
 
 static void
@@ -1446,7 +1519,7 @@ gtk_spin_button_real_change_value (GtkSpinButton *spin,
     gtk_widget_error_bell (GTK_WIDGET (spin));
 }
 
-static void
+static double
 gtk_spin_button_snap (GtkSpinButton *spin_button,
                       double         val)
 {
@@ -1455,7 +1528,7 @@ gtk_spin_button_snap (GtkSpinButton *spin_button,
 
   inc = gtk_adjustment_get_step_increment (spin_button->adjustment);
   if (inc == 0)
-    return;
+    return val;
 
   tmp = (val - gtk_adjustment_get_lower (spin_button->adjustment)) / inc;
   if (tmp - floor (tmp) < ceil (tmp) - tmp)
@@ -1463,7 +1536,21 @@ gtk_spin_button_snap (GtkSpinButton *spin_button,
   else
     val = gtk_adjustment_get_lower (spin_button->adjustment) + ceil (tmp) * inc;
 
-  gtk_spin_button_set_value (spin_button, val);
+  return val;
+}
+
+static void
+gtk_spin_button_changed (GtkEditable   *editable,
+                         GtkSpinButton *spin_button)
+{
+  spin_button->edited = TRUE;
+}
+
+static void
+gtk_spin_button_real_activate (GtkSpinButton *spin_button)
+{
+  if (spin_button->activates_default)
+    gtk_widget_activate_default (GTK_WIDGET (spin_button));
 }
 
 static void
@@ -1472,8 +1559,15 @@ gtk_spin_button_activate (GtkText *entry,
 {
   GtkSpinButton *spin_button = user_data;
 
-  if (gtk_editable_get_editable (GTK_EDITABLE (spin_button->entry)))
-    gtk_spin_button_update (spin_button);
+  if (!gtk_editable_get_editable (GTK_EDITABLE (spin_button->entry)))
+    return;
+
+  const gboolean was_edited = spin_button->edited;
+
+  gtk_spin_button_update (spin_button); /* resets edited */
+
+  if (!was_edited)
+    g_signal_emit (spin_button, spinbutton_signals[ACTIVATE], 0);
 }
 
 static void
@@ -1836,6 +1930,51 @@ gtk_spin_button_new_with_range (double min,
   return GTK_WIDGET (spin);
 }
 
+/**
+ * gtk_spin_button_set_activates_default: (attributes org.gtk.Method.set_property=activates-default)
+ * @spin_button: a `GtkSpinButton`
+ * @activates_default: %TRUE to activate window’s default widget on activation
+ *
+ * Sets whether activating the spin button will activate the default
+ * widget for the window containing the spin button.
+ *
+ * See [signal@Gtk.SpinButton::activate] for what counts as activation.
+ *
+ * Since: 4.14
+ */
+void
+gtk_spin_button_set_activates_default (GtkSpinButton *spin_button,
+                                       gboolean       activates_default)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+
+  activates_default = !!activates_default;
+
+  if (activates_default != spin_button->activates_default)
+    {
+      spin_button->activates_default = activates_default;
+      g_object_notify_by_pspec (G_OBJECT (spin_button), spinbutton_props[PROP_ACTIVATES_DEFAULT]);
+    }
+}
+
+/**
+ * gtk_spin_button_get_activates_default: (attributes org.gtk.Method.get_property=activates-default)
+ * @spin_button: a `GtkSpinButton`
+ *
+ * Retrieves the value set by [method@Gtk.SpinButton.set_activates_default].
+ *
+ * Returns: %TRUE if the spin button will activate the default widget
+ *
+ * Since: 4.14
+ */
+gboolean
+gtk_spin_button_get_activates_default (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->activates_default;
+}
+
 /**
  * gtk_spin_button_set_adjustment: (attributes org.gtk.Method.set_property=adjustment)
  * @spin_button: a `GtkSpinButton`
@@ -1851,6 +1990,8 @@ gtk_spin_button_set_adjustment (GtkSpinButton *spin_button,
 
   if (!adjustment)
     adjustment = gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
+  else if (spin_button->adjustment == adjustment)
+    return;
 
   gtk_spin_button_configure (spin_button,
                              adjustment,
@@ -2082,6 +2223,8 @@ gtk_spin_button_set_value (GtkSpinButton *spin_button,
       if (!return_val)
         gtk_spin_button_default_output (spin_button);
     }
+
+  spin_button->edited = FALSE;
 }
 
 /**
@@ -2395,26 +2538,22 @@ gtk_spin_button_update (GtkSpinButton *spin_button)
   else if (return_val == GTK_INPUT_ERROR)
     error = 1;
 
+  const double lower = gtk_adjustment_get_lower (spin_button->adjustment);
+  const double upper = gtk_adjustment_get_upper (spin_button->adjustment);
+
   if (spin_button->update_policy == GTK_UPDATE_ALWAYS)
     {
-      if (val < gtk_adjustment_get_lower (spin_button->adjustment))
-        val = gtk_adjustment_get_lower (spin_button->adjustment);
-      else if (val > gtk_adjustment_get_upper (spin_button->adjustment))
-        val = gtk_adjustment_get_upper (spin_button->adjustment);
+      val = CLAMP (val, lower, upper);
+
+      if (spin_button->snap_to_ticks)
+        val = gtk_spin_button_snap (spin_button, val);
+
+      gtk_spin_button_set_value (spin_button, val);
     }
-  else if ((spin_button->update_policy == GTK_UPDATE_IF_VALID) &&
-           (error ||
-            val < gtk_adjustment_get_lower (spin_button->adjustment) ||
-            val > gtk_adjustment_get_upper (spin_button->adjustment)))
+  else if (error || val < lower || val > upper)
     {
       gtk_spin_button_value_changed (spin_button->adjustment, spin_button);
-      return;
     }
-
-  if (spin_button->snap_to_ticks)
-    gtk_spin_button_snap (spin_button, val);
-  else
-    gtk_spin_button_set_value (spin_button, val);
 }
 
 GtkText *
@@ -2422,4 +2561,3 @@ gtk_spin_button_get_text_widget (GtkSpinButton *spin_button)
 {
   return GTK_TEXT (spin_button->entry);
 }
-
