@@ -181,7 +181,7 @@ mapped_changed (GdkSurface *surface,
                 GtkWidget  *widget)
 {
   if (!gdk_surface_get_mapped (surface))
-    gtk_widget_hide (widget);
+    gtk_widget_set_visible (widget, FALSE);
 }
 
 static gboolean
@@ -198,8 +198,7 @@ surface_event (GdkSurface *surface,
                GdkEvent   *event,
                GtkWidget  *widget)
 {
-  gtk_main_do_event (event);
-  return TRUE;
+  return gtk_main_do_event (event);
 }
 
 static void
@@ -240,8 +239,7 @@ gtk_tooltip_window_unrealize (GtkWidget *widget)
   g_signal_handlers_disconnect_by_func (window->surface, surface_render, widget);
   g_signal_handlers_disconnect_by_func (window->surface, surface_event, widget);
   gdk_surface_set_widget (window->surface, NULL);
-  gdk_surface_destroy (window->surface);
-  g_clear_object (&window->surface);
+  g_clear_pointer (&window->surface, gdk_surface_destroy);
 }
 
 
@@ -258,7 +256,7 @@ surface_transform_changed_cb (GtkWidget               *widget,
                               const graphene_matrix_t *transform,
                               gpointer                 user_data)
 {
-  GtkTooltipWindow *window = GTK_TOOLTIP_WINDOW (widget);
+  GtkTooltipWindow *window = GTK_TOOLTIP_WINDOW (user_data);
 
   gtk_tooltip_window_relayout (window);
 
@@ -401,41 +399,14 @@ gtk_tooltip_window_new (void)
   return g_object_new (GTK_TYPE_TOOLTIP_WINDOW, NULL);
 }
 
-static void
-update_label_width (GtkLabel *label)
-{
-  const char *text;
-
-  text = gtk_label_get_text (label);
-  if (strchr (text, '\n'))
-    {
-      gtk_label_set_wrap (label, FALSE);
-    }
-  else
-    {
-      int len;
-
-      len = g_utf8_strlen (text, -1);
-
-      gtk_label_set_max_width_chars (label, MIN (len, 50));
-      gtk_label_set_wrap (label, TRUE);
-    }
-}
-
 void
 gtk_tooltip_window_set_label_markup (GtkTooltipWindow *window,
                                      const char       *markup)
 {
   if (markup != NULL)
-    {
-      gtk_label_set_markup (GTK_LABEL (window->label), markup);
-      update_label_width (GTK_LABEL (window->label));
-      gtk_widget_show (window->label);
-    }
-  else
-    {
-      gtk_widget_hide (window->label);
-    }
+    gtk_label_set_markup (GTK_LABEL (window->label), markup);
+
+  gtk_widget_set_visible (window->label, markup != NULL);
 }
 
 void
@@ -443,15 +414,9 @@ gtk_tooltip_window_set_label_text (GtkTooltipWindow *window,
                                    const char       *text)
 {
   if (text != NULL)
-    {
-      gtk_label_set_text (GTK_LABEL (window->label), text);
-      update_label_width (GTK_LABEL (window->label));
-      gtk_widget_show (window->label);
-    }
-  else
-    {
-      gtk_widget_hide (window->label);
-    }
+    gtk_label_set_text (GTK_LABEL (window->label), text);
+
+  gtk_widget_set_visible (window->label, text != NULL);
 }
 
 void
@@ -459,44 +424,27 @@ gtk_tooltip_window_set_image_icon (GtkTooltipWindow *window,
                                    GdkPaintable     *paintable)
 {
   if (paintable != NULL)
-    {
-      gtk_image_set_from_paintable (GTK_IMAGE (window->image), paintable);
-      gtk_widget_show (window->image);
-    }
-  else
-    {
-      gtk_widget_hide (window->image);
-    }
+    gtk_image_set_from_paintable (GTK_IMAGE (window->image), paintable);
+
+  gtk_widget_set_visible (window->image, paintable != NULL);
 }
 
 void
 gtk_tooltip_window_set_image_icon_from_name (GtkTooltipWindow *window,
                                              const char       *icon_name)
 {
+  gtk_widget_set_visible (window->image, icon_name != NULL);
   if (icon_name)
-    {
-      gtk_image_set_from_icon_name (GTK_IMAGE (window->image), icon_name);
-      gtk_widget_show (window->image);
-    }
-  else
-    {
-      gtk_widget_hide (window->image);
-    }
+    gtk_image_set_from_icon_name (GTK_IMAGE (window->image), icon_name);
 }
 
 void
 gtk_tooltip_window_set_image_icon_from_gicon (GtkTooltipWindow *window,
                                               GIcon            *gicon)
 {
+  gtk_widget_set_visible (window->image, gicon != NULL);
   if (gicon != NULL)
-    {
-      gtk_image_set_from_gicon (GTK_IMAGE (window->image), gicon);
-      gtk_widget_show (window->image);
-    }
-  else
-    {
-      gtk_widget_hide (window->image);
-    }
+    gtk_image_set_from_gicon (GTK_IMAGE (window->image), gicon);
 }
 
 void
@@ -525,9 +473,9 @@ gtk_tooltip_window_set_custom_widget (GtkTooltipWindow *window,
       window->custom_widget = g_object_ref (custom_widget);
 
       gtk_box_append (GTK_BOX (window->box), custom_widget);
-      gtk_widget_show (custom_widget);
-      gtk_widget_hide (window->image);
-      gtk_widget_hide (window->label);
+      gtk_widget_set_visible (custom_widget, TRUE);
+      gtk_widget_set_visible (window->image, FALSE);
+      gtk_widget_set_visible (window->label, FALSE);
     }
 }
 
