@@ -45,6 +45,7 @@ typedef struct _TreeAugment TreeAugment;
 
 struct _TreeNode
 {
+  gpointer item;
   GListModel *model;
   GtkTreeListRow *row;
   GtkRbTree *children;
@@ -87,6 +88,7 @@ struct _GtkTreeListRow
   GObject parent_instance;
 
   TreeNode *node; /* NULL when the row has been destroyed */
+  gpointer item;
 };
 
 struct _GtkTreeListRowClass
@@ -99,8 +101,15 @@ static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 static GtkTreeListModel *
 tree_node_get_tree_list_model (TreeNode *node)
 {
-  for (; !node->is_root; node = node->parent)
-    { }
+  if (node->is_root)
+    return node->list;
+
+  for (node = node->parent; !node->is_root; node = node->parent)
+    {
+      /* This can happen during collapsing of a parent node */
+      if (node->children == NULL)
+        return NULL;
+    }
 
   return node->list;
 }
@@ -159,50 +168,6 @@ tree_node_get_n_children (TreeNode *node)
   return child_aug->n_items;
 }
 
-static guint
-tree_node_get_local_position (GtkRbTree *tree,
-                              TreeNode  *node)
-{
-  TreeNode *left, *parent;
-  TreeAugment *left_aug;
-  guint n;
-  
-  left = gtk_rb_tree_node_get_left (node);
-  if (left)
-    {
-      left_aug = gtk_rb_tree_get_augment (tree, left);
-      n = left_aug->n_local;
-    }
-  else
-    {
-      n = 0;
-    }
-
-  for (parent = gtk_rb_tree_node_get_parent (node);
-       parent;
-       parent = gtk_rb_tree_node_get_parent (node))
-    {
-      left = gtk_rb_tree_node_get_left (parent);
-      if (left == node)
-        {
-          /* we are the left node, nothing changes */
-        }
-      else
-        {
-          /* we are the right node */
-          n++;
-          if (left)
-            {
-              left_aug = gtk_rb_tree_get_augment (tree, left);
-              n += left_aug->n_local;
-            }
-        }
-      node = parent;
-    }
-
-  return n;
-}
-
 static guint
 tree_node_get_position (TreeNode *node)
 {
@@ -316,30 +281,15 @@ static GListModel *
 tree_node_create_model (GtkTreeListModel *self,
                         TreeNode         *node)
 {
-  TreeNode *parent = node->parent;
   GListModel *model;
-  GObject *item;
 
-  item = g_list_model_get_item (parent->model,
-                                tree_node_get_local_position (parent->children, node));
-  model = self->create_func (item, self->user_data);
-  g_object_unref (item);
+  model = self->create_func (node->item, self->user_data);
   if (model == NULL)
     node->empty = TRUE;
 
   return model;
 }
 
-static gpointer
-tree_node_get_item (TreeNode *node)
-{
-  TreeNode *parent;
-
-  parent = node->parent;
-  return g_list_model_get_item (parent->model,
-                                tree_node_get_local_position (parent->children, node));
-}
-
 static GtkTreeListRow *
 tree_node_get_row (TreeNode *node)
 {
@@ -351,6 +301,7 @@ tree_node_get_row (TreeNode *node)
     {
       node->row = g_object_new (GTK_TYPE_TREE_LIST_ROW, NULL);
       node->row->node = node;
+      node->row->item = g_object_ref (node->item);
 
       return node->row;
     }
@@ -372,6 +323,9 @@ gtk_tree_list_model_items_changed_cb (GListModel *model,
   guint i, tree_position, tree_removed, tree_added, n_local;
 
   self = tree_node_get_tree_list_model (node);
+  if (self == NULL)
+    return;
+
   n_local = g_list_model_get_n_items (model) - added + removed;
 
   if (position < n_local)
@@ -413,10 +367,12 @@ gtk_tree_list_model_items_changed_cb (GListModel *model,
     }
 
   tree_added = added;
-  for (i = 0; i < added; i++)
+  for (i = added; i-- > 0;)
     {
       child = gtk_rb_tree_insert_before (node->children, child);
       child->parent = node;
+      child->item = g_list_model_get_item (model, position + i);
+      g_assert (child->item);
     }
   if (self->autoexpand)
     {
@@ -459,9 +415,17 @@ gtk_tree_list_model_clear_node (gpointer data)
   TreeNode *node = data;
 
   if (node->row)
-    gtk_tree_list_row_destroy (node->row);
+    {
+      g_object_freeze_notify (G_OBJECT (node->row));
+      gtk_tree_list_row_destroy (node->row);
+    }
 
   gtk_tree_list_model_clear_node_children (node);
+
+  if (node->row)
+    g_object_thaw_notify (G_OBJECT (node->row));
+
+  g_clear_object (&node->item);
 }
 
 static void
@@ -516,6 +480,8 @@ gtk_tree_list_model_init_node (GtkTreeListModel *list,
     {
       node = gtk_rb_tree_insert_after (self->children, node);
       node->parent = self;
+      node->item = g_list_model_get_item (model, i);
+      g_assert (node ->item);
       if (list->autoexpand)
         gtk_tree_list_model_expand_node (list, node);
     }
@@ -596,7 +562,7 @@ gtk_tree_list_model_get_item (GListModel *list,
 
   if (self->passthrough)
     {
-      return tree_node_get_item (node);
+      return g_object_ref (node->item);
     }
   else
     {
@@ -995,16 +961,12 @@ G_DEFINE_TYPE (GtkTreeListRow, gtk_tree_list_row, G_TYPE_OBJECT)
 static void
 gtk_tree_list_row_destroy (GtkTreeListRow *self)
 {
-  g_object_freeze_notify (G_OBJECT (self));
+  self->node = NULL;
 
   /* FIXME: We could check some properties to avoid excess notifies */
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_DEPTH]);
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_EXPANDABLE]);
   g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_EXPANDED]);
-  g_object_notify_by_pspec (G_OBJECT (self), row_properties[ROW_PROP_ITEM]);
-
-  self->node = NULL;
-  g_object_thaw_notify (G_OBJECT (self));
 }
 
 static void
@@ -1071,6 +1033,8 @@ gtk_tree_list_row_dispose (GObject *object)
   if (self->node)
     self->node->row = NULL;
 
+  g_clear_object (&self->item);
+
   G_OBJECT_CLASS (gtk_tree_list_row_parent_class)->dispose (object);
 }
 
@@ -1171,7 +1135,8 @@ gtk_tree_list_row_get_position (GtkTreeListRow *self)
  * of zero, rows corresponding to items of models of direct children
  * of the root model have a depth of 1 and so on.
  *
- * The depth of a row never changes until the row is destroyed.
+ * The depth of a row never changes until the row is removed from its model
+ * at which point it will forever return 0.
  *
  * Returns: The depth of this row
  */
@@ -1227,6 +1192,8 @@ gtk_tree_list_row_set_expanded (GtkTreeListRow *self,
     return;
 
   list = tree_node_get_tree_list_model (self->node);
+  if (list == NULL)
+    return;
 
   if (expanded)
     {
@@ -1279,7 +1246,8 @@ gtk_tree_list_row_get_expanded (GtkTreeListRow *self)
  * This does not mean that the row is actually expanded,
  * this can be checked with [method@Gtk.TreeListRow.get_expanded].
  *
- * If a row is expandable never changes until the row is destroyed.
+ * If a row is expandable never changes until the row is removed
+ * from its model at which point it will forever return %FALSE.
  *
  * Returns: %TRUE if the row is expandable
  */
@@ -1301,6 +1269,9 @@ gtk_tree_list_row_is_expandable (GtkTreeListRow *self)
     return TRUE;
 
   list = tree_node_get_tree_list_model (self->node);
+  if (list == NULL)
+    return FALSE;
+
   model = tree_node_create_model (list, self->node);
   if (model)
     {
@@ -1317,21 +1288,16 @@ gtk_tree_list_row_is_expandable (GtkTreeListRow *self)
  *
  * Gets the item corresponding to this row,
  *
- * The value returned by this function never changes until the
- * row is destroyed.
- *
  * Returns: (nullable) (type GObject) (transfer full): The item
- *   of this row or %NULL when the row was destroyed
+ *   of this row. This function is only marked as nullable for backwards
+ *   compatibility reasons.
  */
 gpointer
 gtk_tree_list_row_get_item (GtkTreeListRow *self)
 {
   g_return_val_if_fail (GTK_IS_TREE_LIST_ROW (self), NULL);
 
-  if (self->node == NULL)
-    return NULL;
-
-  return tree_node_get_item (self->node);
+  return g_object_ref (self->item);
 }
 
 /**
@@ -1371,7 +1337,8 @@ gtk_tree_list_row_get_children (GtkTreeListRow *self)
  * %NULL is returned.
  *
  * The value returned by this function never changes
- * until the row is destroyed.
+ * until the row is removed from its model at which point
+ * it will forever return %NULL.
  *
  * Returns: (nullable) (transfer full): The parent of @self
  */
