@@ -184,7 +184,7 @@ update_window_icon (GtkWindow *window,
     {
       gtk_image_set_from_paintable (GTK_IMAGE (icon), paintable);
       g_object_unref (paintable);
-      gtk_widget_show (icon);
+      gtk_widget_set_visible (icon, TRUE);
 
       return TRUE;
     }
@@ -346,9 +346,9 @@ update_window_buttons (GtkWindowControls *self)
           image = g_object_new (GTK_TYPE_IMAGE,
                                 "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
                                 "icon-name", "window-close-symbolic",
+                                "use-fallback", TRUE,
                                 NULL);
           gtk_widget_add_css_class (button, "close");
-          g_object_set (image, "use-fallback", TRUE, NULL);
           gtk_button_set_child (GTK_BUTTON (button), image);
           gtk_widget_set_can_focus (button, FALSE);
           gtk_actionable_set_action_name (GTK_ACTIONABLE (button),
