@@ -1,5 +1,8 @@
 #! /bin/sh
 
+export LIBGL_ALWAYS_SOFTWARE=1
+export GDK_VULKAN_DEVICE=1
+
 version=$(head -5 meson.build | grep version | sed -e "s/[^']*'//" -e "s/'.*$//")
 release_build_dir="release_build"
 branch=$(git branch --show-current)
@@ -13,7 +16,7 @@ fi
 meson subprojects update gi-docgen
 
 # make the release tarball
-meson setup -Dgtk_doc=true --force-fallback-for gi-docgen ${release_build_dir} || exit
+meson setup -Dintrospection=enabled -Dgtk_doc=true --force-fallback-for gi-docgen ${release_build_dir} || exit
 meson compile -C${release_build_dir} || exit
 meson dist -C${release_build_dir} --include-subprojects || exit
 
